from threading import Timer
from threading import RLock
from threading import Thread
from yangke.base import is_ip_address
from yangke.common.config import logger
import time
import yangke.common.http_client as http_client
import json  # 这里不能用json5替换，因为url请求的post方法接收的json字符串不支持json5，主要是json5生成的key不带双引号会报错
import socket

"""
Status of instances
"""
INSTANCE_STATUS = "UP"
# INSTANCE_STATUS_DOWN = "DOWN"
# INSTANCE_STATUS_STARTING = "STARTING"
# INSTANCE_STATUS_OUT_OF_SERVICE = "OUT_OF_SERVICE"
INSTANCE_STATUS_UNKNOWN = "UNKNOWN"
"""
Default eureka server url.
"""
_DEFAULT_EUREKA_SERVER_URL = "http://127.0.0.1:8761/eureka/"

"""
The timeout seconds that all http request to the eureka server
"""
_DEFAULT_TIME_OUT = 5
"""
Default instance field values
"""
_DEFAULT_INSTNACE_PORT = 9090
_DEFAULT_INSTNACE_SECURE_PORT = 9443
_RENEWAL_INTERVAL_IN_SECS = 30
_DURATION_IN_SECS = 90
_DEFAULT_DATA_CENTER_INFO = "MyOwn"
_DEFAULT_DATA_CENTER_INFO_CLASS = "com.netflix.appinfo.InstanceInfo$DefaultDataCenterInfo"
"""
Default encoding
"""
_DEFAULT_ENCODING = "utf-8"

""" 
This is for the DiscoveryClient, when this strategy is set, get_service_url will random choose one of the UP instance and return its url
This is the default strategy
"""
HA_STRATEGY_RANDOM = 1
"""
This is for the DiscoveryClient, when this strategy is set, get_service_url will always return one instance until it is down
"""
HA_STRATEGY_STICK = 2
"""
This is for the DiscoveryClient, when this strategy is set, get_service_url will always return a new instance if any other instances are up
"""
HA_STRATEGY_OTHER = 3
__cache_registry_clients_lock = RLock()
__cache_key = "default"
__cache_registry_clients = {}


def init_registry_client(eureka_server=_DEFAULT_EUREKA_SERVER_URL,
                         app_name="",
                         instance_id="",
                         instance_host="",
                         instance_ip="",
                         instance_port=_DEFAULT_INSTNACE_PORT,
                         instance_unsecure_port_enabled=True,
                         instance_secure_port=_DEFAULT_INSTNACE_SECURE_PORT,
                         instance_secure_port_enabled=False,
                         countryId=1,  # @deprecaded
                         data_center_name=_DEFAULT_DATA_CENTER_INFO,  # Netflix, Amazon, MyOwn
                         renewal_interval_in_secs=_RENEWAL_INTERVAL_IN_SECS,
                         duration_in_secs=_DURATION_IN_SECS,
                         home_page_url="",
                         status_page_url="",
                         health_check_url="",
                         secure_health_check_url="",
                         vip_adr="",
                         secure_vip_addr="",
                         is_coordinating_discovery_server=False,
                         metadata={}):
    """
    通过RLock()防止注册服务被多线程调用出错
    """
    with __cache_registry_clients_lock:
        client = RegistryClient(eureka_server=eureka_server,
                                app_name=app_name,
                                instance_id=instance_id,
                                instance_host=instance_host,
                                instance_ip=instance_ip,
                                instance_port=instance_port,
                                instance_unsecure_port_enabled=instance_unsecure_port_enabled,
                                instance_secure_port=instance_secure_port,
                                instance_secure_port_enabled=instance_secure_port_enabled,
                                countryId=countryId,
                                data_center_name=data_center_name,
                                renewal_interval_in_secs=renewal_interval_in_secs,
                                duration_in_secs=duration_in_secs,
                                home_page_url=home_page_url,
                                status_page_url=status_page_url,
                                health_check_url=health_check_url,
                                secure_health_check_url=secure_health_check_url,
                                vip_adr=vip_adr,
                                secure_vip_addr=secure_vip_addr,
                                is_coordinating_discovery_server=is_coordinating_discovery_server,
                                metadata=metadata)
        __cache_registry_clients[__cache_key] = client
        client.start()
        return client


def init(eureka_server=_DEFAULT_EUREKA_SERVER_URL,
         regions=[],
         app_name="",
         instance_id="",
         instance_host="",
         instance_ip="",
         instance_port=_DEFAULT_INSTNACE_PORT,
         instance_unsecure_port_enabled=True,
         instance_secure_port=_DEFAULT_INSTNACE_SECURE_PORT,
         instance_secure_port_enabled=False,
         countryId=1,  # @deprecaded
         data_center_name=_DEFAULT_DATA_CENTER_INFO,  # Netflix, Amazon, MyOwn
         renewal_interval_in_secs=_RENEWAL_INTERVAL_IN_SECS,
         duration_in_secs=_DURATION_IN_SECS,
         home_page_url="",
         status_page_url="",
         health_check_url="",
         secure_health_check_url="",
         vip_adr="",
         secure_vip_addr="",
         is_coordinating_discovery_server=False,
         metadata={},
         ha_strategy=HA_STRATEGY_RANDOM):
    """
    注释主程序入口
    """
    # 注册该服务
    registry_client = init_registry_client(eureka_server=eureka_server,
                                           app_name=app_name,
                                           instance_id=instance_id,
                                           instance_host=instance_host,
                                           instance_ip=instance_ip,
                                           instance_port=instance_port,
                                           instance_unsecure_port_enabled=instance_unsecure_port_enabled,
                                           instance_secure_port=instance_secure_port,
                                           instance_secure_port_enabled=instance_secure_port_enabled,
                                           countryId=countryId,
                                           data_center_name=data_center_name,
                                           renewal_interval_in_secs=renewal_interval_in_secs,
                                           duration_in_secs=duration_in_secs,
                                           home_page_url=home_page_url,
                                           status_page_url=status_page_url,
                                           health_check_url=health_check_url,
                                           secure_health_check_url=secure_health_check_url,
                                           vip_adr=vip_adr,
                                           secure_vip_addr=secure_vip_addr,
                                           is_coordinating_discovery_server=is_coordinating_discovery_server,
                                           metadata=metadata)
    # 发现服务
    # discovery_client = init_discovery_client(eureka_server,
    #                                          regions=regions,
    #                                          renewal_interval_in_secs=renewal_interval_in_secs,
    #                                          ha_strategy=ha_strategy)
    discovery_client = None
    return registry_client, discovery_client


"""====================== Registry Client ======================================="""


class RegistryClient:
    """Eureka client for spring cloud"""

    def __init__(self,
                 eureka_server=_DEFAULT_EUREKA_SERVER_URL,
                 app_name="",
                 instance_id="",
                 instance_host="",
                 instance_ip="",
                 instance_port=_DEFAULT_INSTNACE_PORT,
                 instance_unsecure_port_enabled=True,
                 instance_secure_port=_DEFAULT_INSTNACE_SECURE_PORT,
                 instance_secure_port_enabled=False,
                 countryId=1,  # @deprecaded
                 data_center_name=_DEFAULT_DATA_CENTER_INFO,  # Netflix, Amazon, MyOwn
                 renewal_interval_in_secs=_RENEWAL_INTERVAL_IN_SECS,
                 duration_in_secs=_DURATION_IN_SECS,
                 home_page_url="",
                 status_page_url="",
                 health_check_url="",
                 secure_health_check_url="",
                 vip_adr="",
                 secure_vip_addr="",
                 is_coordinating_discovery_server=False,
                 metadata={}):
        assert eureka_server is not None and eureka_server != "", "eureka server must be specified."
        assert app_name is not None and app_name != "", "application name must be specified."
        assert instance_port > 0, "port is unvalid"
        assert isinstance(metadata, dict), "metadata must be dict"
        instance_host = instance_host or ""
        instance_ip = instance_ip or ""
        self.__net_lock = RLock()  # 获取一把锁，貌似也不一定需要
        self.__eureka_servers = eureka_server.split(",")
        self.__instance_host = self.__instance_ip = None

        def try_to_get_client_ip(url):
            if instance_host == "" and instance_ip == "":  # 如果二者都为空
                self.__instance_host = self.__instance_ip = RegistryClient.__get_instance_ip(url)
            elif instance_host != "" and instance_ip == "":  # 如果host不为空，ip为空
                self.__instance_host = instance_host
                if is_ip_address(instance_host):
                    self.__instance_ip = instance_host
                else:
                    self.__instance_ip = RegistryClient.__get_instance_ip(url)
            else:  # 如果直接配置了ip，直接使用ip即可
                self.__instance_host = instance_ip
                self.__instance_ip = instance_ip

        self.__try_all_eureka_server(try_to_get_client_ip)

        mdata = {
            'management.port': str(instance_port)
        }
        mdata.update(metadata)  # 使用metadata中的值更新mdata
        self.__instance = {
            'instanceId': instance_id if instance_id != "" else "%s:%s:%d" % (
                self.__instance_host, app_name.lower(), instance_port),  # 示例id，如果未指定就是 host:app_name:port
            'hostName': self.__instance_host,
            'app': app_name.upper(),
            'ipAddr': self.__instance_ip,
            'port': {
                '$': instance_port,
                '@enabled': str(instance_unsecure_port_enabled).lower()
            },
            'securePort': {
                '$': instance_secure_port,
                '@enabled': str(instance_secure_port_enabled).lower()
            },
            'countryId': countryId,
            'dataCenterInfo': {
                '@class': _DEFAULT_DATA_CENTER_INFO_CLASS,
                'name': data_center_name
            },
            'leaseInfo': {
                'renewalIntervalInSecs': renewal_interval_in_secs,
                'durationInSecs': duration_in_secs,
                'registrationTimestamp': 0,
                'lastRenewalTimestamp': 0,
                'evictionTimestamp': 0,
                'serviceUpTimestamp': 0
            },
            'metadata': mdata,
            'homePageUrl': RegistryClient.__format_url(home_page_url, self.__instance_host, instance_port),
            'statusPageUrl': RegistryClient.__format_url(status_page_url, self.__instance_host, instance_port, "info"),
            'healthCheckUrl': RegistryClient.__format_url(health_check_url, self.__instance_host, instance_port,
                                                          "health"),
            'secureHealthCheckUrl': secure_health_check_url,
            'vipAddress': vip_adr if vip_adr != "" else app_name.lower(),
            'secureVipAddress': secure_vip_addr if secure_vip_addr != "" else app_name.lower(),
            'isCoordinatingDiscoveryServer': str(is_coordinating_discovery_server).lower()
        }

        self.__alive = False
        self.__heart_beat_timer = Timer(renewal_interval_in_secs, self.__heart_beat)  # 定时调用self.__heart_beat函数
        self.__heart_beat_timer.daemon = True  # 设置为守护线程，不会阻塞主线程执行，主线程也不会等待该线程结束

    def __try_all_eureka_server(self, fun):
        """
        对所有eureka服务器执行函数
        如发送心跳、获取服务、注册服务等

        fun 可能是一个url转换函数，从defaultZone中获取真正的eureka server地址
        fun 也可能是一个
        :param fun:
        :return:
        """
        with self.__net_lock:
            untry_servers = self.__eureka_servers  # yml文件中的eureka的defaultZone列表
            tried_servers = []
            ok = False
            while len(untry_servers) > 0:
                url = untry_servers[0].strip()
                try:
                    fun(url)
                except (http_client.HTTPError, http_client.URLError):
                    logger.warning("Eureka server [%s] is down, use next url to try." % url)
                    tried_servers.append(url)
                    untry_servers = untry_servers[1:]
                else:
                    ok = True
                    break
            if len(tried_servers) > 0:
                untry_servers.extend(tried_servers)
                self.__eureka_servers = untry_servers
            if not ok:
                raise http_client.URLError("All eureka servers are down!")

    @staticmethod
    def __format_url(url, host, port, defalut_ctx=""):
        """
        生成微服务的各个页面的访问地址

        :param url: 访问的地址，如果不指定，该方法默认会使用示例的注册ip:port
        :param host:
        :param port:
        :param defalut_ctx:
        :return:
        """
        if url != "":  # 指定了url
            if url.startswith('http'):  # 说明指定了完整地址
                _url = url
            elif url.startswith('/'):  # 说明仅指定了 路由路径
                _url = 'http://%s:%d%s' % (host, port, url)
            else:
                _url = 'http://%s:%d/%s' % (host, port, url)
        else:  # 没有指定url
            _url = 'http://%s:%d/%s' % (host, port, defalut_ctx)
        return _url

    @staticmethod
    def __get_instance_ip(eureka_server):
        """
        获取系统ip，如果eureka_server中使用了ipv6，则返回本地ipv6地址
        :param eureka_server:
        :return:
        """
        url_obj = http_client.parse_url(eureka_server)  # 从defaultZone字符串中提取host和port
        target_ip = url_obj["host"]
        target_port = url_obj["port"]
        if target_port is None:
            if url_obj["schema"] == "http":
                target_port = 80
            else:
                target_port = 443

        if url_obj["ipv6"] is not None:
            target_ip = url_obj["ipv6"]
            socket_family = socket.AF_INET6
        else:
            socket_family = socket.AF_INET

        s = socket.socket(socket_family, socket.SOCK_DGRAM)
        s.connect((target_ip, target_port))
        ip = s.getsockname()[0]
        s.close()
        return ip

    def register(self, status=INSTANCE_STATUS, overriddenstatus=INSTANCE_STATUS_UNKNOWN):
        """
        将 self.__instance字典作为json数据发送给eureka进行注册

        :param status:
        :param overriddenstatus:
        :return:
        """
        self.__instance["status"] = status
        self.__instance["overriddenstatus"] = overriddenstatus
        self.__instance["lastUpdatedTimestamp"] = str(int(time.time() * 1000))
        self.__instance["lastDirtyTimestamp"] = str(int(time.time() * 1000))
        try:
            # 将匿名函数传入__try_all...，这里用lambda函数是为了传入参数url
            self.__try_all_eureka_server(lambda url: _register(url, self.__instance))
        except:
            logger.exception("error!")
        else:
            self.__alive = True

    def start(self):
        logger.debug("start to registry client...")
        self.register()
        self.__heart_beat_timer.start()

    def send_heart_beat(self, overridden_status=""):
        """
        客户端实例发送心跳，会使用全局的 send_heart_beat()方法发送

        :param overridden_status:
        :return:
        """
        try:
            self.__try_all_eureka_server(lambda url: send_heart_beat(url, self.__instance["app"],
                                                                     self.__instance["instanceId"],
                                                                     self.__instance["lastDirtyTimestamp"],
                                                                     status=self.__instance["status"],
                                                                     overriddenstatus=overridden_status))
        except:
            logger.exception("Error!")
            logger.info("Cannot send heartbeat to server, try to register")
            self.register()

    def __heart_beat(self):
        while True:
            logger.debug("sending heart beat to spring cloud server ")
            self.send_heart_beat()
            time.sleep(self.__instance["leaseInfo"]["renewalIntervalInSecs"])


def send_heart_beat(eureka_server, app_name, instance_id, last_dirty_timestamp, status=INSTANCE_STATUS,
                    overriddenstatus=""):
    """
    发送心跳

    :param eureka_server:
    :param app_name:
    :param instance_id:
    :param last_dirty_timestamp:
    :param status:
    :param overriddenstatus:
    :return:
    """
    url = _format_url(eureka_server) + "apps/%s/%s?status=%s&lastDirtyTimestamp=%s" % \
          (app_name, instance_id, status, str(last_dirty_timestamp))
    logger.debug("heartbeat url::" + url)
    if overriddenstatus != "":
        url += "&overriddenstatus=" + overriddenstatus

    req = http_client.Request(url)
    req.get_method = lambda: "PUT"
    http_client.load(req, timeout=_DEFAULT_TIME_OUT)


def _register(eureka_server, instance_dic):
    req = http_client.Request(_format_url(eureka_server) + "apps/%s" % instance_dic["app"])  # 生成请求对象，准备发送
    req.add_header('Content-Type', 'application/json')
    req.get_method = lambda: "POST"
    http_client.load(req, json.dumps({"instance": instance_dic}).encode(_DEFAULT_ENCODING), timeout=_DEFAULT_TIME_OUT)


def _format_url(url: str) -> str:
    """
    在连接url时，确保网址最后一个字符是 "/"
    """
    if url.endswith('/'):
        return url
    else:
        return url + "/"



import os
import pandas as pd
import re

from yangke.common.config import logger
from yangke.common.fileOperate import read_csv_ex


def get_detailed_type_of_data_file(file):
    """
    判断性能试验获得的数据文件类型，是IMP数据、功率数据还是DCS导数或者是其他可以识别处理的文件
    可以处理以下类别的数据：
    1. 有线采集仪采集的数据，返回 imp
    2. 无线采集仪采集的数据，返回 wsn
    3. 功率表采集的功率数据，power
    4. 从pi数据库导出的dcs数据，返回dcs_pi，只支持.xlsx，例如神华国华北京燃机电厂导出的数据
    5. 从SIS系统导出的数据，返回sis
    6. 和利时的点表文件，记录了KKS码对应的点描述，返回kp_hollysys
    7. 和利时的DCS文件，返回dcs_hollysys
    8. 西安热工院DCS系统，返回dcs_tpri
    9. dcs系统导出的数据，dcs1，以陕投商洛电厂的数据格式为例
    10. dcs系统导出的.xls格式的数据，以陕投商洛电厂的数据格式为例，返回dcs2
    11. dcs系统导出的.csv格式数据，以德源府谷电厂3、4号机组为例，返回dcs3
    12. 南京科远dcs系统导出的.csv格式数据，如中煤昔阳电厂DCS系统，返回dcs_sciyon
    13. 西门子SPPA-T3000控制系统导出的数据，如天津IGCC电厂DEH和TCS的数据，返回'SPPA-T3000 and ; sep'
    14. 西门子SPPA-T3000控制系统导出的数据，如果以,分割，则返回'SPPA-T3000 and , sep'

    :param file: 文件名
    :return: power表示功率表原始文件，imp表示imp到处的excel文件，power with out title表示不带表头的功率文件，dcs1是dcs导数的一种类型，
    具体数据文件示例参见yangke/performance/data_process_example_file/<type>_type_file.*
    """
    if os.path.splitext(os.path.basename(file))[0].endswith("修正"):  # 只处理原始文件，不处理修正数据
        return "修正"
    elif os.path.splitext(os.path.basename(file))[0].endswith("汇总"):
        return "汇总"
    ext_ = os.path.splitext(file)[-1].lower()

    if ext_ == ".xlsx":
        data = pd.read_excel(file, header=None, sheet_name=0, usecols=[0, 1])
    elif ext_ == ".csv":
        data = read_csv_ex(file, sep=",", header=None)
    elif ext_ == ".txt":
        data = read_csv_ex(file, sep="\s+", header=None, on_bad_lines='skip')
    else:
        return None

    data.dropna(axis=0, how="all", inplace=True)
    cell00 = str(data.iloc[0, 0]).strip()

    if cell00 is None:
        return None
    elif cell00.startswith("Model"):
        return "power"
    elif cell00.startswith("Store No."):
        return "power with out title"
    elif cell00.startswith("开始时间"):
        return "imp"
    elif cell00.startswith("历史趋势开始于"):
        return "dcs_tpri"
    elif cell00.startswith('$DataFileVersion'):  # 测试版本： $DataFileVersion: 3
        return "dcs_sciyon"
    elif cell00.startswith("时间") and cell00 != "时间段":
        if ext_ == ".xlsx":
            try:
                if len(data.iloc[1][1]) > 5:
                    return "dcs_pi"
                else:
                    return "wsn"
            except:
                return "wsn"
        elif ext_ == ".csv":
            return "sis"
    elif cell00.startswith("Start") or cell00.endswith(".tgd"):  #
        return "dcs1"
    elif re.match("[0-9]{1,4}年[0-9]{1,2}月[0-9]{1,2}日", cell00):  # 匹配yyyy年mm月dd日
        return "dcs2"
    elif cell00 == "PN" or cell00 == "SN":  # SN是一个看不见的sheet中的第一个cell的内容
        return "kp_hollysys"
    elif cell00 == "时间段":
        return "dcs_hollysys"
    elif cell00 == "Time":
        data.dropna(axis=1, how="all", inplace=True)
        if data.shape[1] > 2:
            return "dcs_3"
        else:
            return "unknown"
    elif cell00.__contains__('SPPA-T3000'): # 说明是西门子的SPPA-T3000系统导出的TCS或者DEH的数据
        if "," not in cell00 and ";" in cell00:  # 该数据后缀名是csv，但实际分割负荷可能是;
            return "SPPA-T3000 and ; sep"  #
        else:
            return "SPPA-T3000 and , sep"

    else:
        logger.warning(f"文件未识别：{file}")
        return "unknown"



import pandas as pd
from yangke.common.fileOperate import read_csv_ex, read_excel
from yangke.base import merge_two_dataframes
import numpy as np
data1 = read_excel(r"C:\Users\YangKe\Desktop\京能2022年1号机数据.xlsx", need_merge=True)
data2 = read_excel(r"C:\Users\YangKe\Desktop\华润2022年2号机间冷循环水数据.xlsx", need_merge=True)


# 多列滚动函数
# handle对滚动的数据框进行处理
def handle(x, df, name, n):
    df = df[name].iloc[x:x + n, :]
    print(df)
    return 1


# group_rolling 进行滚动
# n：滚动的行数
# df：目标数据框
# name：要滚动的列名
def group_rolling(n, df, name):
    df_roll = pd.DataFrame({'a': list(range(len(df) - n + 1))})
    df_roll['a'].rolling(window=1).apply(lambda x: handle(int(x[0]), df, name, n), raw=True)


data1.dropna(how='any', axis=0, inplace=True)
data2.dropna(how='any', axis=0, inplace=True)
from yangke.base import interpolate_df

data1 = interpolate_df(data1, time_interval=60)
data2 = interpolate_df(data2, time_interval=60)
# group_rolling(20, data, ['功率'])
gonglv = data['功率']

for i in range(gonglv.shape[0]):
    d = gonglv[i]

print("...")
numbers = [float(eval(f"self.lineEdit_{i}.text()")) for i in range(1, 10)]
np.array(numbers).tofile("test.txt")


from flask import Flask, request, make_response
import hashlib
import xml.etree.ElementTree as ET

app = Flask(__name__)


@app.route('/', methods=['GET', 'POST'])
def wechat_auth():
    if request.method == 'GET':
        token = '5d9d54c798119aab5dfc126ea45432a2'  # 替换为自己在公众号设置中的Token
        data = request.args
        signature = data.get('signature', '')
        timestamp = data.get('timestamp', '')
        nonce = data.get('nonce', '')
        echostr = data.get('echostr', '')
        list = [token, timestamp, nonce]
        list.sort()
        s = ''.join(list).encode('utf-8')
        if (hashlib.sha1(s).hexdigest() == signature):
            return make_response(echostr)
        else:
            return 'Invalid Signature'
    else:
        xml_data = request.stream.read()
        xml_tree = ET.fromstring(xml_data)
        msg_type = xml_tree.find('MsgType').text
        if msg_type == 'text':
            content = xml_tree.find('Content').text
            response_content = chat_with_gpt(content)
            response = generate_text_response(xml_tree, response_content)
            return make_response(response)
        else:
            response_content = '暂不支持该类型消息的回复。'
            response = generate_text_response(xml_tree, response_content)
            return make_response(response)


def chat_with_gpt(text):
    # 调用ChatGPT的API接口，返回对话结果
    pass


def generate_text_response(xml_tree, content):
    # 根据对话结果生成XML格式的回复消息
    pass


if __name__ == "__main__":
    app.run(port=80)



"""
定义全局变量的模块
程序使用的所有全局变量定义在这里
python模块自动是单例模式，因此，在多个py文件中引用该模块，多个py模块中的变量对象是同一个
"""
from yangke.stock.dataset.tushareData import StockData
import argparse
import os
import pickle
from pymysql.cursors import Cursor
from sqlalchemy.engine.base import Engine

namespace = argparse.Namespace()


class GlobalVar:
    def __init__(self, encoding='utf8', data_folder=None, tsd: StockData = None, cmd=None,
                 symbol=None, storage=None):
        self.encoding = encoding
        self.data_folder = data_folder  # 股票数据存储的目录
        self.tsd = tsd  # StockData类的对象，其构造函数会初始化tushare账户的口令
        self.cmd = cmd  # 调用程序的命令
        self.symbol = symbol  # 操作哪只股票，symbol对应股票代码
        self.storage = storage  # 数据的存储方式，默认是mysql，也可以取值为"file"，表示本地存储


class RunState:
    """
    储存一些软件运行状态参数
    """

    def __init__(self, last_update_time=None, stocks=None):
        # -------------- 最后更新的股票列表和更新时间 ----------------
        self.stocks = stocks
        self.last_update_time = last_update_time
        # -------------- 最后更新的股票列表和更新时间 ----------------

        folder = os.path.join(dataDirectory, 'state')
        self.file = os.path.abspath(os.path.join(folder, 'RunState'))

    def load(self):
        if os.path.exists(self.file):
            with open(self.file, 'rb') as f:
                state = pickle.load(f)
        else:
            return self
        return state

    def dump(self):
        os.makedirs(os.path.dirname(self.file), exist_ok=True)
        with open(self.file, 'wb') as f:
            pickle.dump(self, f, 0)

    def __repr__(self):
        return str(self.__dict__)

    def __str__(self):
        return str(self.__dict__)


def test():
    state = RunState()
    state.__setattr__('code', 'updated')  # 可以动态向RunState中添加属性
    print(state)

    state.dump()

    state1 = RunState().load()
    print(state1)



from yangke.common.fileOperate import read_csv_ex


def nn_fit(file, x_titles, y_titles):
    """
    根据file中的数据，以x_titles列为输入，y_titles列为输出，拟合神经网络模型

    :param file:
    :param x_titles:
    :param y_titles:
    :return:
    """
    data = read_csv_ex(file)
    x_df = data[x_titles]
    y_df = data[y_titles]



"""
热力系统图的组件类
"""
from cocos.cocosnode import CocosNode
from pyglet.gl import *


class Boiler(CocosNode):
    def __init__(self, position=(80, 200), scale=1, line_width=2, color=(0, 0, 0)):
        """
        锅炉图标
        """
        super(Boiler, self).__init__()
        self.position = position
        self.scale = scale
        self.line_width = line_width
        self.color = color

        self.in_water_coor = None  # 给水的连接点坐标
        self.main_steam_coor = None  # 主蒸汽的连接点坐标
        self.cold_reheat_steam_coor = None  # 冷再的连接点坐标
        self.heat_reheat_steam_coor = None  # 热再的连接点坐标
        self.draw()

    def move_by(self, dx, dy):
        """
        相对移动

        :param dx:
        :param dy:
        :return:
        """
        self.position = self.position[0] + dx, self.position[1] + dy

    def move_to(self, x, y):
        """
        移动到位置(x,y)

        :param x:
        :param y:
        :return:
        """
        self.position = x, y

    def draw(self):
        # glPushMatrix()
        # self.transform()
        glLineWidth(3)
        glColor4b(0, 0, 0, 100)
        # # glTexCoord2d(0, 0)
        glBegin(GL_LINES)
        glVertex2f(self.position[0] - 40, self.position[1] - 60)
        glVertex2f(self.position[0] + 40, self.position[1] - 60)
        glVertex2f(self.position[0] + 40, self.position[1] - 60)
        glVertex2f(self.position[0] + 40, self.position[1] + 60)
        glVertex2f(self.position[0] + 40, self.position[1] + 60)
        glVertex2f(self.position[0] - 40, self.position[1] + 60)
        glVertex2f(self.position[0] - 40, self.position[1] + 60)
        glVertex2f(self.position[0] - 40, self.position[1] - 60)
        glEnd()
        # #
        # glFlush()
        # glPopMatrix()
        # glRectf(self.position[0]-40, self.position[1]-60, self.position[0]+40, self.position[1]+60)



"""
应城电厂的热耗优化软件，用于优化低压和中压两股抽汽的抽汽量
240,280,350

1、2号机供气量范围
煤耗数据数量
"""
import warnings  # 屏蔽MATPLOTLIBDATA警告

warnings.filterwarnings("ignore", "(?s).*MATPLOTLIBDATA.*", category=UserWarning)

from yangke.common.config import logger, initLogger
from yangke.common.fileOperate import write_func, read_func, write_as_pickle, read_from_pickle, get_last_modified_time
from yangke.base import Line, interpolate_value_complex
import os
from yangke.base import get_settings, plot_3d, get_args, interpolate_2d

import pandas as pd
from yangke.common.qt import QYKFigure
import traceback

import sys
from PyQt5.QtWidgets import QApplication, QMainWindow, QMessageBox, QSplitter, QLabel, QTabWidget, QStatusBar, \
    QVBoxLayout, QFileDialog, QInputDialog, QLineEdit, QDesktopWidget, QHBoxLayout
from PyQt5.QtGui import QFont, QKeyEvent
from PyQt5.QtCore import Qt
import PyQt5.QtCore as QtCore
from yangke.common.qt import YkItem, layout_to_widget, set_menu_bar, YkDataTableWidget, YkInputPanel, YkWindow

# 如果需要打包exe，需要显式引入以下包，打包命令pyinstaller -w heat_power_dispatch.py即可，打包为文件夹运行效果更接近IDE运行结果
# -w是窗口模式打包，不暴露控制台，当-w打包命令行模式运行可能不正确
import yangke.common.loggingHandler

power1_func = None  # 1号机功率计算函数
coal_consume_1_func = None  # 1号机煤耗计算函数
coal_consume_2_func = None  # 2号机煤耗计算函数
hr_func = None  # 总热耗计算函数
fl1_range_points = {}  # 1号机中压抽汽对应的低压抽汽范围
need_update = True
data1 = None
data2 = None  # 2号机 (功率, 四段抽汽量, 热耗)
data3 = None  # 2号机 (功率, 四段抽汽量, 煤耗)


def get_hr1():
    """
    获取1号机组热耗，单位kJ/(kW.h)

    :return:
    """

    return 3800  # 1号机组热耗，假定不变，单位kJ/(kW.h)


def get_fl1_available(fm):
    """
    当中压供汽量给定时，获取低压供汽量调节范围

    :param fm: 中压供汽量
    :return:
    """
    if fm < 0 or fm > 430:
        # logger.error(f"中压抽汽量超范围。{fm} not in [0, 430]")
        print(f"中压抽汽量超范围。{fm} not in [0, 430]")
        exit(0)
    fm_keys = fl1_range_points.keys()
    fm1 = [float(d) for d in fm_keys]
    l1 = [fl1_range_points.get(key)[0] for key in fm_keys]
    h1 = [fl1_range_points.get(key)[1] for key in fm_keys]

    low_range = interpolate_value_complex(fm, x_list=fm1, y_list=l1)
    high_range = interpolate_value_complex(fm, x_list=fm1, y_list=h1)
    return low_range, high_range


def get_fl2_available(p2):
    """
    当2号机功率确定时，其低压供汽范围

    :param p2:
    :return:
    """
    if p2 <= 306.617:  # MW
        return 0, 300
    else:
        return 0, -3.929564 * p2 + 1506.416649


def get_power1(fl, fm):
    """
    获取指定中低压供汽量下的电功率。
    适用于“热定电”运行模式，给定中压供汽需求和低压供汽需求，会有一个确定的发电功率

    :param fl: 低压供汽量
    :param fm: 中压供汽量
    :return:
    """
    # 判断fl和fm的取值范围，不在范围内返回None
    # todo

    # noinspection PyCallingNonCallable
    return float(power1_func(fl, fm))


def get_hr2(power, extract4):
    """
    获取指定电功率和抽汽量的热耗率。
    适用于抽凝供热机组，一般使用再热蒸汽或四抽供热

    :param power: 电功率或机组所带负荷
    :param extract4: 四抽供汽量
    :return:
    """
    # noinspection PyCallingNonCallable
    return float(hr_func(power, extract4))


def get_coal_consume1(fl, fm):
    """
    获取1号机的煤耗

    :param fl:
    :param fm:
    :return:
    """
    # noinspection PyCallingNonCallable
    return float(coal_consume_1_func(fl, fm))


def get_coal_consume2(power2, flow_extract4):
    """
    获取2号机的煤耗

    :param power2: 2号机功率
    :param flow_extract4: 2号机
    :return:
    """
    # noinspection PyCallingNonCallable
    return float(coal_consume_2_func(power2, flow_extract4))


def cal_total_obj(fm1, fl1, p, p_type, fl2, object_func="hr"):
    """
    计算 目标函数

    :param fm1: 1号机中压供汽量
    :param fl1: 1号机低压供汽量
    :param p: 电功率
    :param p_type: 参数p的含义，默认是2号机功率，也可设置为全厂功率，可取值"p2"/"total"
    :param fl2: 2号机低压供汽量
    :param object_func: 目标函数类型，“hr”表示热耗，"coal"表示煤耗
    :return:
    """
    p1 = get_power1(fl=fl1, fm=fm1)
    if p_type == "p2":
        if object_func == "hr":
            hr1 = get_hr1()
            p2 = p
            hr2 = get_hr2(p, fl2)
            y = (p1 * hr1 + p2 * hr2) / (p1 + p2)
        else:
            c1 = get_coal_consume1(fl1, fm1)
            c2 = get_coal_consume2(p, fl2)
            y = (p1 * c1 + p * c2) / (p1 + p)
    else:  # p_type="total"
        if object_func == "hr":
            hr1 = get_hr1()
            p1 = get_power1(fl1, fm1)
            p2 = p - p1
            hr2 = get_hr2(p2, fl2)
            y = (p1 * hr1 + p2 * hr2) / (p1 + p2)
        else:
            c1 = get_coal_consume1(fl1, fm1)
            p1 = get_power1(fl1, fm1)
            p2 = p - p1
            c2 = get_coal_consume2(p2, fl2)
            y = (p1 * c1 + p2 * c2) / (p1 + p2)

    return y


def verify_opt(fl_total, fm1, fl1_range, fl2_range, p, p_type):
    """
    通过穷举校核遗传算法优化结果是否正确，

    :param fl1_range: 1号机的低压供汽量范围
    :param fl2_range: 2号机的低压供汽量范围
    :return:
    """
    import numpy as np
    coal_total = []
    x = []
    break_type = 0
    for fl1 in np.arange(round(fl1_range[0]), round(fl1_range[1]) + 1, 1):
        p1 = get_power1(fl1, fm1)
        coal1 = get_coal_consume1(fl1, fm1)
        fl2 = fl_total - fl1
        if fl2_range[0] <= fl2 <= fl2_range[1]:
            if p_type == "p2":  # 说明是#2负荷分配
                coal2 = get_coal_consume2(p, fl2)
                p_total = p1 + p
                coal_total.append((p1 * coal1 + p * coal2) / p_total)
                x.append(fl2)
            elif p_type == "total":  # 说明是全厂负荷分配
                p2 = p - p1
                if p2 > 350 or p2 < 210:
                    break_type = "p2"
                    continue
                coal2 = get_coal_consume2(p2, fl2)
                coal_total.append((p1 * coal1 + p2 * coal2) / p)
                x.append(fl2)
        elif fl2 < fl2_range[0]:
            break_type = "fl2"
            break  # 因为1号机低压供汽量是从小到大穷举的，如果fl1最小时，fl2取到的最大值仍小于fl2的可取值下限，则无法满足指定的供汽量
        else:
            break_type = "fl2_exceed_max"
            continue
    if len(coal_total) == 0:
        logger.debug("指定参数下没有合适的运行工况")
        break_type = "error 0:指定参数下没有合适的运行工况"
        return break_type, x, coal_total, 0, 0
    opt_coal = min(coal_total)
    opt_fl2 = x[coal_total.index(opt_coal)]
    opt_fl1 = fl_total - opt_fl2
    return break_type, x[::-1], coal_total[::-1], opt_fl1, opt_fl2


def get_optimum(fl_need: float = 200, fm1: float = 240, p: float = 350, p_type="p2", object_func="hr", settings={}):
    """
    获得给定参数下，1号机和2号机的低压供汽量最优值

    :param fl_need: 低压供汽量总需求
    :param fm1: 1号机中压供汽量
    :param p: 电功率
    :param p_type: 参数p的含义，默认是2号机功率，也可设置为全厂功率，可取值"p2"/"total"
    :param settings: 算法设置参数
    :param object_func: 优化目标，“hr”表示优化热耗率，“coal”表示优化煤耗率
    :return:
    """
    fl_need = float(fl_need)
    fm1 = float(fm1)
    p = float(p)

    def ga_func(paras: tuple):
        fl1 = paras[0]
        y = cal_total_obj(fm1=fm1, fl1=fl1, p=p, p_type=p_type, fl2=fl_need - fl1, object_func=object_func)
        return y

    from yangke.sko.GA import GA
    fl_l, fl_h = get_fl1_available(fm1)
    size_pop = settings.get("pop_size") or 20
    gens = settings.get("gens") or 40
    p_mutate = settings.get("p_mutate") or 0.001
    ga = GA(func=ga_func, n_dim=1, lb=[fl_l], ub=[fl_h], size_pop=size_pop, max_iter=gens,
            prob_mut=p_mutate)
    best_x, best_y = ga.run()
    print('best_x:', best_x, '\n', 'best_y:', best_y)
    Y_history = pd.DataFrame(ga.all_history_Y)
    # fig, ax = plt.subplots(2, 1)
    # ax[0].plot(Y_history.index, Y_history.values, '.', color='red')
    # Y_history.min(axis=1).cummin().plot(kind='line')
    # plt.show()
    print(1)

    if p_type == "p2":  # 给定的是2号机负荷时
        fl1_range = get_fl1_available(fm1)  # 定值
        fl1 = best_x[0]
        p2 = p  # 定值
        fl2_range = get_fl2_available(p2)  # 定值
        break_type, x_list, coal_total_list, fl1, fl2 = verify_opt(fl_need, fm1, fl1_range, fl2_range, p,
                                                                   p_type)  # 更新最优结果
    else:
        print(2)
        fl1_range = get_fl1_available(fm1)
        fl1 = best_x[0]
        p1 = get_power1(fl1, fm1)
        p2 = p - p1
        fl2_range = get_fl2_available(p2)
        fl2 = fl_need - fl1
        if fl2 > fl2_range[1]:  # 如果fl2超范围，则将fl2_range适当放大，在此求解最优值，这里fl2和fl2_range是相互影响的
            fl2_range = (fl2_range[0], (fl2_range[1] + fl2) / 2)
        break_type, x_list, coal_total_list, fl1, fl2 = verify_opt(fl_need, fm1, fl1_range, fl2_range, p,
                                                                   p_type)  # 更新最优结果
        print(3)
        if isinstance(break_type, str) and "error" in break_type:
            fl1 = best_x
            fl2 = fl_need - fl1
            return best_x, best_y, Y_history, fl1, fl2, x_list, coal_total_list, break_type
        p1 = get_power1(fl1, fm1)
        p2 = p - p1
        fl2_range = get_fl2_available(p2)
        print(4)
        flag = 0
        while fl2 < fl2_range[0] or fl2 > fl2_range[1]:
            fl2_range = (fl2_range[0], (fl2_range[1] + fl2) / 2)
            break_type, x_list, coal_total_list, fl1, fl2 = verify_opt(fl_need, fm1, fl1_range, fl2_range, p,
                                                                       p_type)  # 更新最优结果
            flag = flag + 1
            if flag > 10:
                break
        print(5)
    print(6)
    if isinstance(break_type, str) and "error" in break_type:  # 说明无法同时满足指定的热电负荷
        if fl_need > fl1_range[1] + fl2_range[1]:
            break_type = "error: 低压供汽量过大，无法满足"
        fl1 = best_x
        fl2 = fl_need - fl1

    # 返回值意义
    # best_x, best_y遗传算法优化得到的fl1和目标函数值的最优结果，但可能超出允许取值范围
    # Y_history遗传算法优化过程中的种群个体的目标函数值信息
    # fl1, fl2将优化结果限定到允许取值范围内得到的优化结果，即这两个参数时最终的优化结果
    # x_list, coal_total_list是2号机低压供汽量和对应的全场煤耗的数据，是在fl1和fl2允许取值范围内的所有情况
    # break_type是提示信息，0表示正常，否则会有相应提示
    print(7)
    return best_x, best_y, Y_history, fl1, fl2, x_list, coal_total_list, break_type


def get_拐点(x_list, y_list, y0):
    """
    找出y=y0与x_list和y_list确定的折线的交点，本项目中，只可能有一个交点

    :param x_list:
    :param y_list:
    :param x:
    :return:
    """
    for i in range(1, len(x_list) - 1):
        point1 = (x_list[i], y_list[i])
        point2 = (x_list[i + 1], y_list[i + 1])
        line1 = Line(point1=point1, point2=point2)
        line2 = Line(k=0, b=y0)
        cross_point, is_cross = line1.cross_point(line2)
        if is_cross:
            if x_list[i] < cross_point[0] < x_list[i + 1]:  # 如果交点位于point1和point2之间，则返回
                return cross_point  # 本项目中最多一个交点，找到则返回
    return None


# ===========================界面部分=======================================

def read_fun_data():
    """
    判断是否需要读入data.xlsx文件
    :return:
    """
    try:
        global power1_func
        power1_func = read_func("power1_func.dat")
        global coal_consume_1_func
        coal_consume_1_func = read_func("coal_consume_1_func.dat")
        global hr_func
        hr_func = read_func("hr_func.dat")
        global coal_consume_2_func
        coal_consume_2_func = read_func("coal_consume_2_func.dat")
        global fl1_range_points
        fl1_range_points = read_from_pickle("fl_range_points.dat")
        return True
    except:
        print("未找到函数文件power1_func.dat")
        return False


class MainWindow(YkWindow):
    def __init__(self):
        self.data_file = None
        self.fm1 = 240  # 1号机中压供汽，已知值
        self.p2 = 300  # 2号机负荷，已知值
        self.fl_total = 240  # 低压供汽总需求，已知值
        self.p_total = 350  # 第二种优化类型中的已知值，此时，p2是未知的
        self.display = {}  # 显示设置
        self.algorithm = {}  # 算法设置
        self.result = {}  # 优化结果，包含键有"fl1", "fl2", "hr_aver", "history"
        self.digits = 2
        super(MainWindow, self).__init__()  # 调用super方法时，会自动调用init_ui方法，因此需要先定义在init_ui方法中使用到的变量
        read_fun_data()

    def init_ui(self):
        YkWindow.不知道有什么用但是有用删除就闪退 = self.init_ui
        if not os.path.exists(os.path.join(os.path.dirname(__file__), "ui/ui_menu.yaml")):
            QMessageBox.warning(self,
                                "警告",
                                "找不到UI设置文件，请检查相关yaml文件路径是否正确！",
                                QMessageBox.Ok)
            sys.exit(0)
        # ----------------------------菜单栏------------------------------
        set_menu_bar(self, from_file=os.path.join(os.path.dirname(__file__), "ui/ui_menu.yaml"))  # 设置菜单栏

        self.root_splitter = QSplitter(Qt.Vertical, self)

        self.splitter = QSplitter(Qt.Horizontal, self)  # 主面板分为左右两部分
        self.splitter_l = QSplitter(Qt.Vertical, self)
        self.splitter_r = QSplitter(Qt.Vertical, self.splitter)  # 右边部分是有一个分割面板，竖直分割
        self.tab = QTabWidget()
        self.splitter_rt = QSplitter(Qt.Horizontal, self.splitter_r)  # 右边最上面又是一个分割面板

        logger.debug("初始化tableWidget")
        self.table_widget = YkDataTableWidget(from_file=os.path.join(os.path.dirname(__file__), "ui/table_data.yaml"))
        self.splitter_rt.addWidget(QLabel("#1功率响应曲面"))
        self.splitter_rt.addWidget(QLabel("#2热耗响应曲面"))
        self.splitter_r.addWidget(self.splitter_rt)
        self.splitter_r.addWidget(QLabel("遗传种群个体分布"))
        self.splitter_r.addWidget(QLabel("最优个体随遗传代数变化"))
        self.tab.addTab(self.splitter_r, "图像")
        self.tab.addTab(self.table_widget, "数据")
        self.splitter_l.addWidget(YkInputPanel(parent=self,
                                               from_file=os.path.join(os.path.dirname(__file__),
                                                                      "ui/ui_data.yaml"),
                                               domain="calculate2"))
        self.result_panel = self.get_result_panel()
        self.splitter_l.addWidget(self.result_panel)
        self.result_panel.setHidden(True)
        self.splitter.addWidget(self.splitter_l)
        self.splitter.addWidget(self.tab)
        self.splitter.setSizes([500, 700])
        h_box = QHBoxLayout()
        h_box.addStretch()
        title_label = QLabel("华能应城电厂全厂热电负荷优化分配智能调度系统")
        title_label.setFont(QFont("楷体", 18))
        title_label.setStyleSheet("color:blue")
        h_box.addWidget(title_label)
        h_box.addStretch()
        self.root_splitter.addWidget(layout_to_widget(h_box))
        self.root_splitter.addWidget(self.splitter)
        self.root_splitter.setSizes([10, 890])
        self.setCentralWidget(self.root_splitter)

        self.setGeometry(0, 0, 1200, 900)
        self.setWindowTitle("应城电厂热电负荷分配优化")
        self.statusBar1: QStatusBar = self.statusBar()
        self.statusBar1.showMessage('就绪')
        self.statusBar1.addPermanentWidget(QLabel("西安热工研究院有限公司＆华能应城热电有限责任公司                     "
                                                  "        ©2020 TPRI. All Rights Reserved."))

    def show_result(self):
        """
        显示优化结果

        :return:
        """

        # 遗传算法最优个体变化图
        fig = QYKFigure(x=self.result.get('history').index,
                        y=self.result.get('history').min(axis=1).cummin(),
                        title="遗传算法最优个体",
                        ylim=None)
        # 遗传算法种群个体分布图
        scatter_fig = QYKFigure(x=self.result.get('history').index,
                                y=self.result.get('history').values, fig_type="scatter",
                                title="种群个体",
                                ylim=None)
        if self.display.get("y_min") is not None:
            fig.axes.set_ylim(bottom=float(self.display.get("y_min")))
            scatter_fig.axes.set_ylim(bottom=float(self.display.get("y_min")))
        if self.display.get("y_max") is not None:
            fig.axes.set_ylim(top=float(self.display.get("y_max")))
            scatter_fig.axes.set_ylim(top=float(self.display.get("y_max")))
        self.splitter_r.replaceWidget(1, fig)
        self.splitter_r.replaceWidget(2, scatter_fig)

        fl1_range = get_fl1_available(self.fm1)  # 定值
        fl1 = self.result.get("fl1")
        fl2 = self.result.get("fl2")
        fm1 = self.fm1  # 定值
        p1 = get_power1(fl1, fm1)
        coal1 = get_coal_consume1(fl1, fm1)
        hr1 = get_hr1()  # 定值
        if self.p2 is not None:  # 给定的是2号机负荷时
            p2 = self.p2  # 定值
        else:
            p2 = self.p_total - p1
        fl2_range = get_fl2_available(p2)  # 定值
        hr2 = get_hr2(p2, fl2)
        coal2 = get_coal_consume2(p2, fl2)
        p_total = p1 + p2
        coal_total_opt = (p1 * coal1 + p2 * coal2) / p_total
        self.result.update({"coal_aver": coal_total_opt})
        self.result_panel = self.get_result_panel()
        self.splitter_l.replaceWidget(1, self.result_panel)
        self.result_panel.setHidden(False)
        self.table_widget.set_value("#2低压供汽范围.下限", round(fl2_range[0], self.digits))
        self.table_widget.set_value("#2低压供汽范围.上限", round(fl2_range[1], self.digits))
        self.table_widget.set_value("#1号机.低压供汽量", round(fl1, self.digits))
        self.table_widget.set_value("#1号机.中压供汽量", round(fm1, self.digits))
        self.table_widget.set_value("#1号机.功率", round(p1, self.digits))
        # self.table_widget.setValue("#1号机.热耗", hr1)
        self.table_widget.set_value("#1号机.煤耗", round(coal1, self.digits))

        self.table_widget.set_value("#2号机.功率", round(p2, self.digits))
        self.table_widget.set_value("#2号机.4抽供汽量", round(fl2, self.digits))
        self.table_widget.set_value("#2号机.热耗", round(hr2, self.digits))
        self.table_widget.set_value("#2号机.煤耗", round(coal2, self.digits))

        self.table_widget.set_value("全厂（最优工况下）.总低压供汽量", round(self.fl_total, self.digits))
        self.table_widget.set_value("全厂（最优工况下）.全厂总负荷", round(p_total, self.digits))
        self.table_widget.set_value("全厂（最优工况下）.全厂平均热耗", round((p1 * hr1 + p2 * hr2) / p_total, self.digits))
        self.table_widget.set_value("全厂（最优工况下）.全厂平均煤耗", round(coal_total_opt, self.digits))
        # 计算1号机单独供汽的工况
        fl1 = self.fl_total
        p1 = get_power1(fl1, fm1)
        coal1 = get_coal_consume1(fl1, fm1)
        hr2 = get_hr2(p2, 0)
        coal2 = get_coal_consume2(p2, 0)
        p_total = p1 + p2
        coal_total_if_only_1 = (p1 * coal1 + p2 * coal2) / p_total

        if fl1 < fl1_range[0]:
            self.table_widget.set_value("备注", "1号机单独供汽工况下，低压供汽量小于已知的下限值，可能超出调节范围")
        elif fl1 > fl1_range[1]:
            self.table_widget.set_value("备注", "1号机单独供汽工况下，低压供汽量大于已知的上限值，可能超出调节范围")
        else:
            increase = (coal_total_if_only_1 - coal_total_opt) / coal_total_opt * 100
            self.table_widget.set_value("备注", "1号机单独供汽煤耗较最优工况增大 " +
                                        str(round(increase, 2)) + "%")
        self.table_widget.set_value("全厂（1号机单独供汽）.总低压供汽量", round(self.fl_total, self.digits))
        self.table_widget.set_value("全厂（1号机单独供汽）.全厂总负荷", round(p_total, self.digits))
        self.table_widget.set_value("全厂（1号机单独供汽）.全厂平均热耗", round((p1 * hr1 + p2 * hr2) / p_total, self.digits))
        self.table_widget.set_value("全厂（1号机单独供汽）.全厂平均煤耗", round(coal_total_if_only_1, self.digits))

        self.table_widget.set_value("#1低压供汽范围.下限", round(fl1_range[0], self.digits))
        self.table_widget.set_value("#1低压供汽范围.上限", round(fl1_range[1], self.digits))

        x_list = self.result.get("x_list")
        break_type = self.result.get("break_type")
        coal_total_list = self.result.get("coal_total_list")
        if len(x_list) == 1:
            if break_type == "fl2":
                self.table_widget.set_value("备注", "低压供汽需求量接近1号机低压供汽量下限，建议只使用1号机进行供汽")
            elif break_type == "p2":
                self.table_widget.set_value("备注", "#2功率超出调节范围")
            elif "error" in break_type:
                self.table_widget.set_value("备注", break_type)
            x_list.append(x_list[0] + 0.01)
            coal_total_list = coal_total_list * 2
        elif len(x_list) == 0:  # 因为寻优开始时已经判断了低压供汽总需求和1号机低压供汽量下限的大小关系，这里不会为0
            self.table_widget.set_value("备注", "无法同时满足热负荷和电负荷")
            QMessageBox.warning(self,
                                "警告",
                                "无法同时满足指定的热负荷和电负荷",
                                QMessageBox.Ok)
            return
        fig1 = QYKFigure(x_list, coal_total_list, fig_type="curve", title="全厂煤耗变化曲线", xlabel="#2低压抽汽量")
        self.table_widget.setSpan(14, 0, 12, 6)
        self.table_widget.setCellWidget(14, 0, fig1)
        x_temp = self.result.get("revert_fl2")
        if x_temp is not None and x_temp != 0:
            fig1.scatter(x_temp, coal_total_list[0])

    def get_result_panel(self):
        """
        构建结果显示面板

        :return:
        """
        if len(self.result) == 0:
            return QLabel("待求解")
        v_box = QVBoxLayout()
        size = get_settings("size", setting_file=os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"))
        v_box.addStretch()
        v_box.addWidget(YkItem("最优低压供汽#1", round(self.result.get("fl1"), 2),
                               unit=["t/h", "kg/s"], unit_selected="t/h",
                               size=size))
        v_box.addWidget(YkItem("最优低压供汽#2", round(self.result.get("fl2"), 2),
                               unit=["t/h", "kg/s"], unit_selected="t/h",
                               size=size))
        v_box.addWidget(YkItem("最优平均煤耗", round(self.result.get("coal_aver"), 2),
                               unit=["g/(kW.h)"], unit_selected="g/(kW.h)",
                               size=size))
        # v_box.addWidget(YkItem("蝶阀开度", ))
        return layout_to_widget(v_box)

    def btn_clicked(self):
        sender = self.sender()
        panel = sender.parent()
        if sender.text() == "应用":
            if panel.name == "calculate":
                values, units = panel.get_values_and_units()
                self.fm1 = values[0]
                self.fl_total = values[1]
                self.p2 = values[2]
            elif panel.name == "calculate2":
                values, units = panel.get_values_and_units()
                self.fm1 = values[0]
                self.fl_total = values[1]
                self.p_total = values[2]
            elif panel.name == "display":
                values, units = panel.get_values_and_units()
                self.display.update({"need_pop_fig": values[0], "method1": values[1],
                                     "method2": values[2], "pop_fig_type": values[3],
                                     "y_min": float(values[4]), "y_max": float(values[5])})
            elif panel.name == "algorithm":
                values, units = panel.get_values_and_units()
                self.algorithm.update({"pop_size": int(values[0]), "gens": int(values[1]),
                                       "p_mutate": float(values[2]), "p_cross": float(values[3]),
                                       "select_method": values[4]})
            self.statusBar().showMessage("就绪")
        elif sender.text() == "寻优":
            if hr_func is None:
                QMessageBox.warning(self,
                                    "警告",
                                    "请先设置数据文件！设置方法：【文件】->【设置数据文件】",
                                    QMessageBox.Ok)
                return
            current_index = self.tab.currentIndex()
            self.tab.setCurrentIndex(0)
            logger.debug("test2")
            # --------------------读入优化计算参数，并判断参数是否合理--------------------------------------
            values, units = panel.get_values_and_units()
            self.fm1 = float(values[0])
            self.fl_total = float(values[1])
            fl1_range = get_fl1_available(self.fm1)
            if self.fl_total < fl1_range[0]:
                QMessageBox.warning(self,
                                    "警告",
                                    f"低压供汽需求（{self.fl_total}）小于1号机最低供汽量（{fl1_range[0]}）,请重新设置输入参数！",
                                    QMessageBox.Ok)
                return
            if panel.name == "calculate":
                self.p2 = float(values[2])
                if self.p2 < 210:
                    QMessageBox.warning(self,
                                        "警告",
                                        f"#2功率（{self.p2}）小于2号机允许最小负荷（210）,请重新设置输入参数！",
                                        QMessageBox.Ok)
                    return
                elif self.p2 > 350:
                    QMessageBox.warning(self,
                                        "警告",
                                        f"#2功率（{self.p2}）大于2号机允许最大负荷（350）,请重新设置输入参数！",
                                        QMessageBox.Ok)
                    return
                self.p_total = None
            elif panel.name == "calculate2":
                self.p_total = float(values[2])
                self.p2 = None
            # --------------------读入优化计算参数，并判断参数是否合理--------------------------------------
            if panel.name == "calculate":
                bestX, bestY, y_his, fl1_true, fl2_true, x_list, coal_list, break_type = get_optimum(
                    fl_need=self.fl_total, fm1=self.fm1, p=self.p2,
                    object_func="coal", settings=self.algorithm)
                self.result.update({"fl1": float(fl1_true), "fl2": float(fl2_true), "revert_fl2": 0,
                                    "x_list": x_list, "coal_total_list": coal_list,
                                    "break_type": break_type, "history": y_his, "type": 1})
            else:  # if panel.name == "calculate2":
                bestX, bestY, y_his, fl1_true, fl2_true, x_list, coal_list, break_type = get_optimum(
                    fl_need=self.fl_total, fm1=self.fm1,
                    p=self.p_total, p_type="total",
                    object_func="coal", settings=self.algorithm)
                self.result.update({"fl1": float(fl1_true), "fl2": float(fl2_true), "revert_fl2": 0,
                                    "x_list": x_list, "coal_total_list": coal_list,
                                    "break_type": break_type, "history": y_his, "type": 1})
            if len(x_list) > 0:
                拐点 = get_拐点(x_list, coal_list, coal_list[0])
                if 拐点 is not None:
                    self.result.update({"revert_fl2": 拐点[0]})
            self.show_result()
            self.tab.setCurrentIndex(current_index)
            self.statusBar().showMessage("就绪")
        elif sender.text() == "确定":
            if panel.name == "algorithm":
                values, units = panel.get_values_and_units()
                self.algorithm.update({"pop_size": int(values[0]), "gens": int(values[1]),
                                       "p_mutate": float(values[2]), "p_cross": float(values[3]),
                                       "select_method": values[4]})
            elif panel.name == "display":
                values, units = panel.get_values_and_units()
                self.display.update({"need_pop_fig": values[0], "method1": values[1],
                                     "method2": values[2], "pop_fig_type": values[3],
                                     "y_min": float(values[4]), "y_max": float(values[5])})
                # 如果有计算结果，就在不计算的情况下，直接更新一次显示结果
                if len(self.result) > 0:
                    self.show_result()

    def init_model(self, debug: bool = True):
        """
        读取excel数据文件，获得原始煤耗、热耗、电功率等数据，并生成插值计算模型

        :param debug:
        :return:
        """
        if not need_update:
            return
        try:
            global power1_func
            num_1_file = self.data_file or r"E:\2020\科研项目\应城电厂热电分配\data.xlsx"
            modified_time = get_last_modified_time(file=num_1_file)
            modified_time_1 = read_from_pickle("last_modified_time.dat")
            if modified_time != modified_time_1:
                write_as_pickle("last_modified_time.dat", modified_time)
            # ================================ 加载1号机功率数据==========================================
            logger.debug("加载1号机功率数据")
            # 默认用第一行作为列索引，使用index_col=0设置用第一列作为行索引
            data = pd.read_excel(num_1_file, sheet_name="1号机功率", index_col=0)
            data = data.dropna(how="all")  # 丢弃所有元素都为NAN的行
            data = data.dropna(how="all", axis=1)  # 丢弃所有元素都为NAN的列
            index_name_col = data.columns
            index_name_row = data.index
            flow_middle, flow_low, power = [], [], []  # 初始化三个数组，用于存放x,y,z，分别对应中压供汽流量...
            for fl in index_name_row:
                for fm in index_name_col:
                    if pd.notna(data[fm][fl]):
                        flow_low.append(fl)
                        flow_middle.append(fm)
                        power.append(data[fm][fl])

            if modified_time == modified_time_1:
                logger.debug("加载1号机功率插值函数")
                power1_func = read_func("power1_func.dat")
            else:
                logger.debug("生成1号机功率插值函数")
                self.statusBar1.showMessage("生成1号机功率插值函数...")
                logger.debug(f"{flow_low}, {flow_middle}, {power}")
                power1_func = interpolate_2d(flow_low, flow_middle, power, method="rbf")  # 生成插值模型
                logger.debug("2")
                write_func("power1_func.dat", power1_func)
                logger.debug("3")

            if debug:
                plot_3d(flow_low, flow_middle, power, method="rbf", projection=False)
            else:
                logger.debug("绘制功率响应曲面")
                fig1 = plot_3d(flow_low, flow_middle, power, method="rbf", projection=False, backend="pyqt5")
                if len(self.splitter_rt.children()) >= 2:
                    logger.debug("替换功率响应曲面")
                    self.splitter_rt.replaceWidget(0, fig1)
                else:
                    logger.debug("添加功率响应曲面")
                    self.splitter_rt.addWidget(fig1)
            # ================================ 加载1号机功率数据==========================================

            # ============================== 生成指定中压抽汽量下对应的低压抽汽量的取值范围=================================
            logger.debug("确定1号机低压供汽量取值范围")
            for fm in flow_middle:
                temp_series = data[:][fm].dropna().index
                # 因为小数涉及到精度问题，当用其作为字典的键时，最好转换为字符串
                fl1_range_points.update({str(fm): [temp_series.min(), temp_series.max()]})
            write_as_pickle("fl_range_points.dat", fl1_range_points)
            # ============================== 生成指定中压抽汽量下对应的低压抽汽量的取值范围=================================

            # =================================加载1号机煤耗数据===============================================
            logger.debug("加载1号机煤耗数据")
            # 默认用第一行作为列索引，使用index_col=0设置用第一列作为行索引
            data = pd.read_excel(num_1_file, sheet_name="1号机煤耗", index_col=0)
            data = data.dropna(how="all")  # 丢弃所有元素都为NAN的行
            data = data.dropna(how="all", axis=1)  # 丢弃所有元素都为NAN的列
            index_name_col = data.columns
            index_name_row = data.index
            flow_middle, flow_low, coal_consume = [], [], []  # 初始化三个数组，用于存放x,y,z，分别对应中压供汽流量...
            for fl in index_name_row:
                for fm in index_name_col:
                    if pd.notna(data[fm][fl]):
                        flow_low.append(fl)
                        flow_middle.append(fm)
                        coal_consume.append(data[fm][fl])
            self.statusBar().showMessage("生成1号机煤耗插值函数...")
            global coal_consume_1_func
            if modified_time == modified_time_1:
                coal_consume_1_func = read_func("coal_consume_1_func.dat")
            else:
                coal_consume_1_func = interpolate_2d(flow_low, flow_middle, coal_consume, method="rbf")  # 生成插值模型
                write_func("coal_consume_1_func.dat", coal_consume_1_func)
            if debug:
                plot_3d(flow_low, flow_middle, coal_consume, method="rbf", projection=False)
            # else:
            #     fig1 = plot_3d(flow_low, flow_middle, coal_consume, method="rbf", projection=False, backend="pyqt5")
            #     if len(self.splitter_rt.children()) >= 2:
            #         self.splitter_rt.replaceWidget(0, fig1)
            #     else:
            #         self.splitter_rt.addWidget(fig1)
            # =================================加载1号机煤耗数据===============================================

            # =================================加载2号机热耗数据===============================================
            logger.debug("加载2号机热耗数据")
            flow_extract4, power, heat_rate = [], [], []
            data = pd.read_excel(num_1_file, sheet_name="2号机热耗", index_col=0)
            data = data.dropna(how="all")  # 丢弃所有元素都为NAN的行
            data = data.dropna(how="all", axis=1)  # 丢弃所有元素都为NAN的列
            index_name_col = data.columns
            index_name_row = data.index
            for fe in index_name_row:
                for po in index_name_col:
                    if pd.notna(data[po][fe]):
                        flow_extract4.append(fe)
                        power.append(po)
                        heat_rate.append(data[po][fe])
            self.statusBar().showMessage("生成2号机热耗插值函数...")
            global hr_func, data2
            data2 = (power, flow_extract4, heat_rate)
            if modified_time == modified_time_1:
                hr_func = read_func("hr_func.dat")
            else:
                hr_func = interpolate_2d(power, flow_extract4, heat_rate, method="rbf")
                write_func("hr_func.dat", hr_func)
            if debug:
                plot_3d(power, flow_extract4, heat_rate, method="rbf", projection=False)
            else:
                fig1 = plot_3d(power, flow_extract4, heat_rate, method="rbf", projection=False, backend="pyqt5")
                if len(self.splitter_rt.children()) >= 2:
                    self.splitter_rt.replaceWidget(1, fig1)
                else:
                    self.splitter_rt.addWidget(fig1)
            # =================================加载2号机热耗数据===============================================

            # =================================加载2号机煤耗数据===============================================
            logger.debug("加载2号机煤耗数据")
            flow_extract4, power, coal_consume = [], [], []
            data = pd.read_excel(num_1_file, sheet_name="2号机煤耗", index_col=0)
            data = data.dropna(how="all")  # 丢弃所有元素都为NAN的行
            data = data.dropna(how="all", axis=1)  # 丢弃所有元素都为NAN的列
            index_name_col = data.columns
            index_name_row = data.index
            for fe in index_name_row:
                for po in index_name_col:
                    if pd.notna(data[po][fe]):
                        flow_extract4.append(fe)
                        power.append(po)
                        coal_consume.append(data[po][fe])
            self.statusBar().showMessage("生成2号机煤耗插值函数...")
            global coal_consume_2_func, data3
            data3 = (power, flow_extract4, coal_consume)
            if modified_time == modified_time_1:
                coal_consume_2_func = read_func("coal_consume_2_func.dat")
            else:
                coal_consume_2_func = interpolate_2d(power, flow_extract4, coal_consume, method="rbf")
                write_func("coal_consume_2_func.dat", coal_consume_2_func)
            # if debug:
            #     plot_3d(power, flow_extract4, coal_consume, method="rbf", projection=False)
            # else:
            #     fig1 = plot_3d(power, flow_extract4, coal_consume, method="rbf", projection=False, backend="pyqt5")
            #     if len(self.splitter_rt.children()) >= 2:
            #         self.splitter_rt.replaceWidget(1, fig1)
            #     else:
            #         self.splitter_rt.addWidget(fig1)
            # =================================加载2号机煤耗数据===============================================
            self.statusBar().showMessage("就绪")
        except Exception:
            logger.debug("初始化计算模型错误")
            self.statusBar().showMessage("初始化计算模型错误！")
            traceback.print_exc()

    def set_data_file(self):
        directory = None
        file = QFileDialog.getOpenFileName(parent=self, caption='选择数据文件', directory=directory)
        file = file[0]
        current_index = self.tab.currentIndex()
        self.tab.setCurrentIndex(0)
        if file is not None and os.path.exists(file):
            self.data_file = os.path.abspath(file)
            global need_update
            need_update = True
            self.statusBar().showMessage(f"数据文件更新：{file}")
            self.init_model(False)
            # start_threads(self.init_model, args_list=(False,))  # 子线程无法更新主线程的界面内容
        self.tab.setCurrentIndex(current_index)
        self.statusBar1.showMessage("就绪")

    def set_algorithm(self):
        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "algorithm")
        if self.display is not None and len(self.display) == len(input_panel.findChildren(YkItem)):
            t = self.display
            values = [t.get("pop_size"), t.get("gens"), t.get("p_mutate"), t.get("p_cross"),
                      t.get("select_method")]
            input_panel.set_values(values=values)
        self.splitter_l.replaceWidget(0, input_panel)
        self.statusBar1.showMessage("就绪")

    def set_display(self):
        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "display")
        if self.display is not None and len(self.display) == len(input_panel.findChildren(YkItem)):
            t = self.display
            values = [t.get("need_pop_fig"), t.get("method1"), t.get("method2"), t.get("pop_fig_type"),
                      t.get("y_min"), t.get("y_max")]
            input_panel.set_values(values=values)
        self.splitter_l.replaceWidget(0, input_panel)
        self.statusBar1.showMessage("就绪")

    def set_calculate(self):
        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "calculate")
        input_panel.set_values([self.fm1, self.fl_total, self.p2])
        self.splitter_l.replaceWidget(0, input_panel)
        self.statusBar1.showMessage("就绪")

    def set_calculate2(self):
        input_panel = YkInputPanel(self, os.path.join(os.path.dirname(__file__), "ui/ui_data.yaml"), "calculate2")
        input_panel.set_values([self.fm1, self.fl_total, self.p_total])
        self.splitter_l.replaceWidget(0, input_panel)
        self.statusBar1.showMessage("就绪")

    def single_calculate(self):
        def _single_calculate(table_widget: YkDataTableWidget):
            print("debug ...")
            if hr_func is None:
                QMessageBox.warning(self,
                                    "警告",
                                    "请先设置数据文件！设置方法：【文件】->【设置数据文件】",
                                    QMessageBox.Ok)
                return
            fm1 = float(table_widget.get_value("给定参数.#1中压供汽量"))
            fl1 = float(table_widget.get_value("给定参数.#1低压供汽量"))
            fl_total = float(table_widget.get_value("给定参数.低压供汽总需求"))
            p2 = float(table_widget.get_value("给定参数.#2功率"))

            p1 = get_power1(fl1, fm1)
            hr1 = get_hr1()
            coal1 = get_coal_consume1(fl1, fm1)

            fl2 = fl_total - fl1
            if (fl2 < 0):
                QMessageBox.warning(self,
                                    "警告",
                                    "#1低压供汽量大于低压供汽总需求",
                                    QMessageBox.Ok)
                return
            hr2 = get_hr2(p2, fl2)
            coal2 = get_coal_consume2(p2, fl2)

            coal_total = (p1 * coal1 + p2 * coal2) / (p1 + p2)
            hr_total = (p1 * hr1 + p2 * hr2) / (p1 + p2)

            table_widget.set_value("计算结果.#1功率", p1)
            table_widget.set_value("计算结果.#1热耗", hr1)
            table_widget.set_value("计算结果.#1煤耗", coal1)
            table_widget.set_value("计算结果.#1中压供汽", fm1)
            table_widget.set_value("计算结果.#1低压供汽", fl1)
            table_widget.set_value("计算结果.#2功率", p2)
            table_widget.set_value("计算结果.#2热耗", hr2)
            table_widget.set_value("计算结果.#2煤耗", coal2)
            table_widget.set_value("计算结果.#2四抽供汽", fl2)
            table_widget.set_value("计算结果.全厂总热耗", hr_total)
            table_widget.set_value("计算结果.全厂总煤耗", coal_total)

        # 因为YkDataTableWidget中使用了按钮事件，需要首先把事件方法绑定到类上，否则按钮的connect将报错
        YkDataTableWidget.single_calculate = _single_calculate
        self.table_widget_single = YkDataTableWidget(from_file=os.path.join(os.path.dirname(__file__),
                                                                            "ui/table_data_single_calculate.yaml"),
                                                     **{"single_calculate": _single_calculate})

        self.tab.addTab(self.table_widget_single, "")
        qua = len(self.tab.children()[1])
        self.tab.setTabText(qua - 1, f"单工况计算{qua - 2}")
        self.tab.setCurrentIndex(qua - 1)
        self.statusBar1.showMessage("就绪")


def get_valve_openness(fl2):
    """
    根据2号低压供汽流量计算2号机低压液控阀开度

    :param fl2:
    :return:
    """
    o = (fl2 - 2.18380941107531) / 4.75368946518066
    if o < 0:
        o = 0
    return o


def command_calculate(fm1, fl_total, p2=None, p_total=None, type_cal="0"):
    """
    如果是命令行调用该程序

    :param fm1:
    :param fl_total:
    :param p2:
    :param p_total:
    :param type_cal:
    :return:
    """
    import json
    if not read_fun_data():
        result = {"type": type_cal, "opt_fl1": 0, "opt_fl2": 0, "opt_coal": 0, "revert_fl2": 0,
                  "opt_p1": 0, "opt_p2": 0, "opt_coal1": 0, "opt_coal2": 0, "break_type": "error:1初始化函数失败"}
        result = json.dumps(result, ensure_ascii=False)
        print(result)
        return result
    if type_cal == "1":
        p = p2
        p_type = "p2"
    elif type_cal == "2":
        p = p_total
        p_type = "total"
    best_x, best_y, Y_history, fl1, fl2, x_list, coal_total_list, break_type = get_optimum(fl_need=fl_total,
                                                                                           fm1=fm1, p=p,
                                                                                           p_type=p_type,
                                                                                           object_func="coal")

    if isinstance(break_type, str) and "error" in break_type:
        result = {"type": type_cal, "opt_fl1": 0, "opt_fl2": 0, "opt_coal": 0, "revert_fl2": 0,
                  "opt_p1": 0, "opt_p2": 0, "opt_coal1": 0, "opt_coal2": 0, "break_type": break_type}
        result = json.dumps(result, ensure_ascii=False)
        print(result)
        return result
    coal1 = get_coal_consume1(fl1, fm1)
    hr1 = get_hr1()
    p1 = get_power1(fl1, fm1)
    if type_cal == "1":
        p_total = p1 + p2
    elif type_cal == "2":
        p2 = p_total - p1
    hr2 = get_hr2(p2, fl2)
    coal2 = get_coal_consume2(p2, fl2)

    coal_total = (p1 * coal1 + p2 * coal2) / (p1 + p2)
    hr_total = (p1 * hr1 + p2 * hr2) / (p1 + p2)
    result = {"type": type_cal, "opt_fl1": fl1, "opt_fl2": fl2, "opt_coal": coal_total,
              "valve": get_valve_openness(fl2), "revert_fl2": 0,
              "opt_p1": p1, "opt_p2": p2, "opt_coal1": coal1, "opt_coal2": coal2, "break_type": break_type}
    if len(x_list) > 0:
        拐点 = get_拐点(x_list, coal_total_list, coal_total_list[0])
        if 拐点 is not None:
            result.update({"revert_fl2": 拐点[0]})
    result = json.dumps(result, ensure_ascii=False)
    print(result)
    return result


print("start")
if __name__ == "__main__":
    args = {'kv': [{'short': 'data_file', 'default': None, 'description': '数据文件路径'},
                   {'short': 'p2', 'default': None, 'description': '#2功率'},
                   {'short': 'p_total', 'default': None, 'description': '全厂总负荷'},
                   {'short': 'fl_total', 'default': None, 'description': '低压供气总需求'},
                   {'short': 'fm1', 'default': None, 'description': '#1中压抽汽量'},
                   {'short': 'log_level', 'default': 30, 'description': '日志输出级别'},
                   {'short': 't', 'long': 'type', 'default': None,
                    'description': '优化算法计算类型，1-#2功率优化：需要指定(p2, fm1, fl_total)，'
                                   '2-全场负荷优化：需要指定(p_total, fm1, fl_total)'},
                   ],
            'k': [{'short': 'c', 'long': 'command', 'default': False, 'description': '运行模式'}]}
    args = get_args(args_name=args)  # 返回的是字典，返回值不分类
    type_cal = args.type
    p_total = None
    p2 = None
    logger.setLevel(int(args.log_level))
    print("version 1.0.0")
    if type_cal is None or type_cal == "0":
        app = QApplication(sys.argv)
        # app.setFont(QFont("Microsoft YaHei", 12))
        # app.setStyleSheet("font-size: 20px")
        w1 = MainWindow()
        w1.setFont(QFont("Microsoft YaHei", 12))
        sys.exit(app.exec_())
    else:
        fm1 = float(args.fm1)
        fl_total = float(args.fl_total)
        data_file = args.exe_file
        if type_cal == "1":
            p2 = float(args.p2)
            assert p2 and fm1 and fl_total
        elif type_cal == "2":
            p_total = float(args.p_total)
            print(f"type_cal={type_cal},p_total={p_total},fm1={fm1},fl_total={fl_total}")
            assert p_total and fm1 and fl_total
    command_calculate(fm1=fm1, fl_total=fl_total, p_total=p_total, p2=p2, type_cal=type_cal)



from yangke.common.qt import (YkWindow, run_app, logger, QApplication, YkFoldableWidget, QWidget, QLabel, QComboBox,
                              QLineEdit, YkItem, YkInputPanel)
from yangke.game.gf import Step, Steps, Frame, NotExist, Exist, Region, AnchorRegion, RectRegion, Position, Offset
from yangke.base import start_threads, stop_threads


class StepWidget(YkItem):
    def __init__(self, idx, op, target, judge, condition, wait_method):
        self.step = Step(op, target, judge, condition, wait_method)
        code = ["press", "double-press", "left-click", "right-click"]  # QComboBox只支持添加元素为字符串的类型
        # self.op_widget = QComboBox()
        # self.op_widget.addItems(code)
        # self.op_widget.setCurrentIndex(0)

        self.op_widget = YkItem(["press", "click", "right click", "double click"],
                                value="",
                                unit=YkItem(label=["重复", "等待"], value=["直到", "无"], unit=["无", "存在", "不存在"],
                                            size=[10, 20, 20],
                                            margins=(10, 0, 10, 0)),
                                size=[20, 20, 50],
                                margins=(10, 0, 10, 0))
        unit = '<button on-click="remove_step">移除步骤</button>'
        super().__init__(f"步骤{idx}", value=self.op_widget, unit=unit, size=[20, 200, 50])
        # self.target_widget = QLineEdit("")
        # self.judge_widget = QComboBox(self)
        # self.judge_widget.addItems(["until", "无"])
        # self.condition_widget = QComboBox(self)
        # self.condition_widget.addItems(["Exist", "NotExist"])


class MainWindow(YkWindow):
    def __init__(self):
        super().__init__()
        self.add_input_panel("ui/ui_panel.yaml")
        self.thread = None
        self.running = False
        self.set_status_bar_label("天谕")
        self.frame: Frame | None = None
        self.add_input_panel(domain="自定义步骤")
        panel: YkInputPanel = self.panels.get("自定义步骤")
        item = StepWidget(1, "press", "R", None, None, None)
        panel.insert_item(1, item)
        self.add_content_tab(YkItem(), "存在条件设置")

    def add_step(self):
        panel: YkInputPanel = self.panels.get("自定义步骤")
        idx = panel.get_items_count()
        panel.insert_item(idx, StepWidget(idx, "press", "R", None, None, None))

    def init(self):
        settings = self.get_value_of_panel(need_dict=True, need_unit=False)
        role = settings.get("游戏角色名").strip()
        self.frame = Frame(role)
        # self.frame = Frame(role, sim_mode="大漠插件-remote", sim_info={
        #     "display": "gdi2",
        #     "mouse": "dx.mouse.position.lock.api|dx.mouse.position.lock.message|dx.mouse.clip.lock.api|dx.mouse.input.lock.api|dx.mouse.state.api|dx.mouse.api|dx.mouse.cursor",
        #     "keypad": "windows",
        #     "public": "",
        #     "mode": 11,
        #     "key": "mmdlljafd8c5cbf193e99306e9eb61ceb5bd44",
        #     "add_key": "ewdSEKFZP",
        #     "guard_type": "memory4",
        #     "port": 8765,
        # })
        self.frame.init_chat(anchor="ui/ZhaoMu.png", anchor_region=AnchorRegion(0, "anchor.y1", "anchor.x1", -10),
                             channels=["世界", "团队", "队伍", "附近", "阵营", "地区"])
        suc = self.frame.init_task(anchor="任务", find_region=RectRegion(left=-350, top=300, right=-1, bottom=-500),
                                   anchor_region=AnchorRegion("anchor.x1", "anchor.y1", -10, "anchor.y1+400"))
        if not suc:
            self.frame.init_task(anchor="驻地", find_region=RectRegion(left=-350, top=300, right=-1, bottom=-500),
                                 anchor_region=AnchorRegion("anchor.x1-100", "anchor.y1", -10, "anchor.y1+400"))

        # self.frame.init_time(region=(-170, 0, -2, 30))
        self.frame.init_time(region=(-25, 5, -2, 25))
        # self.frame.turn_direction_to("青蛙", region=Region(width_child=1000, height_child=800))

    def ShengWangRenWu(self):
        self.init()
        time = self.frame.get_time()
        steps_寻路至平海镇布告栏处 = Steps(steps=[
            # 打开人物属性面板
            Step("press", "ctrl+c", "until", Exist("ui/RWSX.png"), wait_method="repeat"),

            # 切换到声望面板
            Step("click", Offset(x_offset=326, y_offset=18), "until",
                 Exist("门派",
                       region=AnchorRegion(x1="anchor.x1-337", y1="anchor.y1+32",
                                           x2="anchor.x1-143", y2="anchor.y1+69")),
                 wait_method="repeat"),

            # 折叠门派声望
            Step("click", Offset(x_offset=0, y_offset=0), "until",
                 Exist("萤川郡",
                       region=AnchorRegion(x1="anchor1.x1-4", y1="anchor.y1+53",
                                           x2="anchor1.x1+80", y2="anchor.y1+100")),
                 wait_method="repeat"),

            # 展开莹川郡声望
            Step("click", Offset(x_offset=0, y_offset=0), "until",
                 Exist("平海镇",
                       region=AnchorRegion(x1="anchor1.x1-16", y1="anchor.y1+40",
                                           x2="anchor1.x1+80", y2="anchor.y1+100")),
                 wait_method="repeat"),

            # 点击平海镇声望
            Step("click", Offset(x_offset=0, y_offset=0), "until",
                 Exist("平海镇",
                       region=AnchorRegion(x1="anchor1.x1+250", y1="anchor.y1-100",
                                           x2="anchor1.x1+380", y2="anchor.y1-55")),
                 wait_method="repeat"),

            # 点击查看声望玩法
            Step("click", Offset(x_offset=468, y_offset=119), "until",
                 Exist("平海镇声望玩法",
                       region=AnchorRegion(x1="anchor1.x1+100", y1="anchor.y1-210",
                                           x2="anchor.x1+300", y2="anchor.y1-160")),
                 wait_method="repeat"),

            # 点击平海镇布告栏导航到平海镇布告栏处
            Step("click", Offset(53, 218), "until",
                 Exist("ui/PingHaiZhenBuGaoLan.png",
                       region=Region(win_frame=self.frame, align="center", width_child=600, height_child=400)),
                 wait_method=None),

            # 点击Esc退出布告栏，寻路完成
            Step("press", "esc", "until", NotExist("ui/PingHaiZhenBuGaoLan.png",
                                                   region=Region(win_frame=self.frame, align="center", width_child=600,
                                                                 height_child=400)),
                 wait_method="repeat"),

        ], debug=True)

        # 判断是否有任务
        ...

        # 接任务
        steps_收集炼金资料 = Steps(steps=[
            Step("click", )
        ])
        self.thread = start_threads(self.frame.run_steps, args_list=[steps_寻路至平海镇布告栏处])

        self.frame.get_text_position("与柏宁玛士荣工作板对话")

    def 寻路至仙姑声望(self):
        steps_寻路至平海镇布告栏处 = Steps(steps=[
            # 打开人物属性面板
            Step("press", "ctrl+c", "until", Exist("ui/RWSX.png"), wait_method="repeat"),

            # 切换到声望面板
            Step("click", Offset(x_offset=326, y_offset=18), "until",
                 Exist("门派",
                       region=AnchorRegion(x1="anchor.x1-337", y1="anchor.y1+32",
                                           x2="anchor.x1-143", y2="anchor.y1+69")),
                 wait_method="repeat"),

            # 折叠门派声望
            Step("click", Offset(x_offset=0, y_offset=0), "until",
                 Exist("苏澜郡",
                       region=AnchorRegion(x1="anchor1.x1-4", y1="anchor.y1+5",
                                           x2="anchor1.x1+80", y2="anchor.y1+100")),
                 wait_method="repeat"),

            # 展开苏澜郡声望
            Step("click", Offset(x_offset=0, y_offset=0), "until",
                 Exist("仙菇",
                       region=AnchorRegion(x1="anchor1.x1-16", y1="anchor.y1+170",
                                           x2="anchor1.x1+80", y2="anchor.y1+230")),
                 wait_method="repeat"),

            # 点击仙菇声望
            Step("click", Offset(x_offset=0, y_offset=0), "until",
                 Exist("仙菇",
                       region=AnchorRegion(x1="anchor1.x1+250", y1="anchor.y1-100",
                                           x2="anchor1.x1+380", y2="anchor.y1-55")),
                 wait_method="repeat"),

            # 点击查看声望玩法
            Step("click", Offset(x_offset=468, y_offset=119), "until",
                 Exist("仙菇声望玩法",
                       region=AnchorRegion(x1="anchor1.x1+100", y1="anchor.y1-210",
                                           x2="anchor.x1+300", y2="anchor.y1-160")),
                 wait_method="repeat"),

            # 点击柏宁玛士荣布告栏导航到平海镇布告栏处
            Step("click", Offset(64, 219), "until",
                 Exist("ui/BoNingMaShiRongBuGaoLan.png",
                       region=Region(win_frame=self.frame, align="center", width_child=600, height_child=400)),
                 wait_method=None),

            # 点击Esc退出布告栏，寻路完成
            Step("press", "esc", "until", NotExist("ui/BoNingMaShiRongBuGaoLan.png",
                                                   region=Region(win_frame=self.frame, align="center", width_child=600,
                                                                 height_child=400)),
                 wait_method="repeat"),

        ], debug=True)

    def run(self):
        self.init()
        settings = self.get_value_of_panel(need_dict=True, need_unit=False)
        key = settings.get("打怪技能按键")
        role = settings.get("游戏角色名").strip()
        mode = settings.get("打怪模式")
        freq = float(settings.get("按键频率"))
        # self.frame = Frame(role)
        # link_pos = self.frame.task.get_text_link_pos_global("打开公会")
        # self.frame.show_region(link_pos)
        #
        # self.frame.left_click(*link_pos.get_center(), offset=(16, 0))
        # self.frame.dm.unbind_window()
        if mode == "无脑打怪":
            steps_重复按键 = Steps(
                steps=[
                    Step("press", key, None, None),
                ]
            )
        else:
            steps_重复按键 = Steps(
                steps=[
                    Step("press", key, "until",
                         NotExist("竹林偷伐者", last_time=20, region=Region(align="center", width_child=600)),
                         wait_method="repeat"),
                    Step("double-press", "space", "until",
                         Exist("竹林偷伐者", last_time=60, interval=10, region=Region(align="center", width_child=600)),
                         wait_method="repeat"),
                ]
            )
        self.thread = start_threads(self.frame.run_steps_forever, args_list=[steps_重复按键, freq])
        self._input_panel.get_button("运行").setDisabled(True)
        self._input_panel.get_button("停止").setDisabled(False)

    def stop(self):
        stop_threads(self.thread)
        self.running = False
        logger.debug(f"停止挂机")
        self._input_panel.get_button("停止").setDisabled(True)
        self._input_panel.get_button("运行").setDisabled(False)

    def remove_step(self):
        ...


run_app(MainWindow)






# 官方提供的resnet实现位于 from torchvision.models import resnet
def 官方提供的各种网络模型():
    import torchvision.models as model
    model.alexnet()
    model.googlenet()
    model.densenet121()
    model.vgg11()
    model.mnasnet1_0()
    model.inception_v3()

    model.shufflenet_v2_x1_0()
    model.squeezenet1_1()
    model.mobilenet_v2()
#
#
# class BasicBlock(nn.Module):
#     """
#     https://www.bilibili.com/video/BV14E411H7Uw?from=search&seid=12206417067392459630
#
#     """
#     expansion = 1
#
#     def __init__(self, in_channel, out_channel, stride=1, downsapmle=None):
#         super(BasicBlock, self).__init__()
#         self.conv1 = nn.Conv2d(in_channels=in_channel, out_channels=out_channel,
#                                kernel_size=3, stride=stride, padding=1, bias=False)
#         self.bn1 = nn.BatchNorm2d(out_channel)
#         self.relu = nn.ReLU()
#         self.conv2 = nn.Conv2d(in_channels=out_channel, out_channels=out_channel,
#                                kernel_size=3, stride=1, padding=1, bias=False)
#         self.bn2 = nn.BatchNorm2d(out_channel)
#         self.downsample = downsapmle
#
#     def forward(self, x):
#         identity = x
#         if self.downsample is not None:
#             identity = self.downsample(x)
#
#         out = self.conv1(x)
#         out = self.bn1(out)
#         out = self.relu(out)
#
#         out = self.conv2(out)
#         out = self.bn2(out)
#
#         out = out + identity
#         out = self.relu(out)
#
#         return out
#
#
# class Bottleneck(nn.Module):
#     expansion = 4
#
#     def __init__(self, in_channel, out_channel, stride=1, downsample=None):
#         super(Bottleneck, self).__init__()
#         self.conv1 = nn.Conv2d(in_channels=in_channel, out_channels=out_channel,
#                                kernel_size=1, stride=1, bias=False)
#         self.bn1 = nn.BatchNorm2d(out_channel)
#         # --------------------------------------------------------
#         self.conv2 = nn.Conv2d(in_channels=out_channel, out_channels=out_channel,
#                                kernel_size=3, stride=stride, bias=False, padding=1)
#         self.bn2 = nn.BatchNorm2d(out_channel)
#         # --------------------------------------------------------
#         self.conv3 = nn.Conv2d(in_channels=out_channel, out_channels=out_channel * self.expansion,
#                                kernel_size=1, stride=1, bias=False)
#         self.bn3 = nn.BatchNorm2d(out_channel * self.expansion)
#         self.relu = nn.ReLU(inplace=True)
#         self.downsample = downsample
#
#     def forward(self, x):
#         identity = x
#         if self.downsample is not None:
#             identity = self.downsample(x)
#
#         out = self.conv1(x)
#         out = self.bn1(out)
#         out = self.relu(out)
#
#         out = self.conv2(out)
#         out = self.bn2(out)
#         out = self.relu(out)
#
#         out = self.conv3(out)
#         out = self.bn3(out)
#
#         out = out + identity
#         out = self.relu(out)
#
#         return out
#
#
# class ResNet(nn.Module):
#     def __init__(self, block, blocks_num, num_classes=1000, include_top=True):
#         super(ResNet, self).__init__()
#         self.include_top = include_top
#         self.in_channel = 64
#
#         self.conv1 = nn.Conv2d(3, self.in_channel, kernel_size=7, stride=2,
#                                padding=3, bias=False)
#         self.bn1 = nn.BatchNorm2d(self.in_channel)
#         self.relu = nn.ReLU(inplace=True)
#         self.maxpool = nn.MaxPool2d(kernel_size=3, stride=2, padding=1)
#
#         self.layer1 = self._make_layer(block, 64, blocks_num[0])
#         self.layer2 = self._make_layer(block, 128, blocks_num[1], stride=2)
#         self.layer2 = self._make_layer(block, 256, blocks_num[2], stride=2)
#         self.layer2 = self._make_layer(block, 512, blocks_num[3], stride=2)
#
#         if self.include_top:
#             self.avgpool = nn.AdaptiveAvgPool2d((1, 1))
#             self.fc = nn.Linear(512 * block.expansion, num_classes)
#         for m in self.modules():
#             if isinstance(m, nn.Conv2d):
#                 nn.init.kaiming_normal_(m.weight, mode='fan_out', nonlinearity='relu')
#
#     def _make_layer(self, block, channel, block_num, stride=1):
#         """
#
#         :param block:
#         :param channel: ResNet各个分区中第一层的深度
#         :param block_num: 长度为4的列表，对应没个分区中的block数量
#         :param stride: 默认步长
#         :return:
#         """
#         downsample = None
#         if stride != 1 or self.in_channel != channel * block.expansion:  # 如果步长不为1或者输出输出的深度不一致
#             downsample = nn.Sequential(  # 用于50，101，152的shortcut，将深度翻四倍，高和宽不变
#                 nn.Conv2d(self.in_channel, channel * block.expansion, kernel_size=1, stride=stride, bias=False),
#                 nn.BatchNorm2d(channel * block.expansion)
#             )
#
#         layers = []
#         layers.append(block(self.in_channel, channel, downsample=downsample, stride=stride))
#         self.in_channel = channel * block.expansion
#
#         for _ in range(1, block_num):
#             layers.append(block(self.in_channel, channel))
#
#         return nn.Sequential(*layers)
#
#     def forward(self, x):
#         x = self.conv1(x)
#         x = self.bn1(x)
#         x = self.relu(x)
#         x = self.maxpool(x)
#
#         x = self.layer1(x)
#         x = self.layer2(x)
#         x = self.layer3(x)
#         x = self.layer4(x)
#
#         if self.include_top:
#             x = self.avgpool(x)
#             x = torch.flatten(x, 1)
#             x = self.fc(x)
#         return x
#
#
# def resnet18(num_classes=1000, include_top=True):
#     return ResNet(BasicBlock, [2, 2, 2, 2], num_classes=num_classes, include_top=include_top)
#
#
# def resnet34(num_classes=1000, include_top=True):
#     return ResNet(BasicBlock, [3, 4, 6, 3], num_classes=num_classes, include_top=include_top)
#
#
# def resnet50(num_classes=1000, include_top=True):
#     return ResNet(Bottleneck, [3, 4, 6, 3], num_classes=num_classes, include_top=include_top)
#
#
# def resnet101(num_classes=1000, include_top=True):
#     return ResNet(Bottleneck, [3, 4, 23, 3], num_classes=num_classes, include_top=include_top)
#
#
# def resnet152(num_classes=1000, include_top=True):
#     return ResNet(Bottleneck, [3, 8, 36, 3], num_classes=num_classes, include_top=include_top)



import torch
import torch.nn as nn
from torchvision.models.resnet import resnet50
import torchvision as tv


class Conv2d(nn.Module):
    """
    这个网络主要由一系列的1×1和3×3的卷积层组成，每个卷积层后都会跟一个BN层和一个LeakyReLU层，
    这里将这三个封装为新的卷积层
    """

    def __init__(self, in_channels, out_channels, kernal_size, stride, padding):
        super(Conv2d, self).__init__()
        self.conv = nn.Sequential(
            nn.Conv2d(in_channels=in_channels, out_channels=out_channels,
                      kernel_size=kernal_size, stride=stride, padding=padding),
            nn.BatchNorm2d(num_features=out_channels),
            nn.LeakyReLU()
        )

    def forward(self, x):
        return self.conv(x)


class Residual(nn.Module):
    """
    Darknet-53中的残差模块，参考https://blog.csdn.net/qq_37541097/article/details/81214953#commentBox
    """

    def __init__(self, in_channels):
        super(Residual, self).__init__()
        self.r = nn.Sequential(
            nn.Conv2d(in_channels=in_channels, out_channels=in_channels // 2, kernel_size=1, stride=1, padding=0),
            nn.Conv2d(in_channels=in_channels // 2, out_channels=in_channels, kernel_size=3, stride=1, padding=1)
        )

    def forward(self, x):
        return x + self.r(x)


class DownSample(nn.Module):
    """
    降采样层，网络结构块之间的卷积层
    """

    def __init__(self, in_channels, kernal_size=3, stride=2, padding=1):
        super(DownSample, self).__init__()
        self.downsample = Conv2d(in_channels=in_channels, out_channels=in_channels, kernal_size=kernal_size,
                                 stride=stride, padding=padding)


def block_times(block, times):
    """
    重复block模块times次
    :param block:
    :param times:
    :return:
    """
    blocks = nn.Sequential()
    for i in range(times):
        blocks.add_module(block)
    return blocks


class ConvSet(nn.Module):
    def __init__(self, in_channels, out_channels):
        super(ConvSet, self).__init__()
        self.conv = nn.Sequential(
            Conv2d(in_channels=in_channels, out_channels=out_channels, kernal_size=1, stride=1, padding=0),
            Conv2d(out_channels, out_channels, 3, 1, 0),
            Conv2d(out_channels, out_channels * 2, 1, 1, 0),
            Conv2d(out_channels * 2, out_channels * 2, 3, 1, 1),
            Conv2d(out_channels * 2, out_channels, 1, 1, 0)
        )

    def forward(self, x):
        return self.conv(x)


class Darknet52(nn.Module):
    """
    这是Darknet53层不包括最后平均池化层之后三层的部分，一共52个卷积层。这里的卷积层计数不算残差层内的卷积层，

    Darknet52可用于构建Dark53和Yolov3网络结构，这是这两个结构的共有部分。
    """

    def __init__(self, in_channels):
        super(Darknet52, self).__init__()
        self.d52 = nn.Sequential(
            Conv2d(in_channels, 32, 3, 1, 1),  # in_channel为图片通道数，一般都是3
            Conv2d(32, 64, 3, 2, 1),

            # 1x
            Conv2d(64, 32, 1, 1, 0),
            Conv2d(32, 64, 3, 1, 1),  # 这里的padding数量都是计算出来的
            Residual(64),

            Conv2d(64, 128, 3, 2, 1),  # 降采样的同时提升通道数
        )
        # 后面涉及到重复的块，这里使用循环分别添加不同的层
        for i in range(2):
            self.d52.add_module(Conv2d(128, 64, 1, 1, 0))
            self.d52.add_module(Conv2d(64, 128, 3, 1, 1))
            self.d52.add_module(Residual(128))

        self.d52.add(Conv2d(128, 256, 3, 2, 1))

        # 8x
        for i in range(8):
            self.d52.add_module(Conv2d(256, 128, 1, 1, 0))
            self.d52.add_module(Conv2d(128, 258, 3, 1, 1))
            self.d52.add_module(Residual(256))

        self.d52.add_module(Conv2d(256, 512, 3, 2, 1))

        # 8x
        for i in range(8):
            self.d52.add_module(Conv2d(512, 256, 1, 1, 0))
            self.d52.add_module(Conv2d(256, 512, 3, 1, 1))
            self.d52.add_module(Residual(512))

        self.d52.add_module(Conv2d(512, 1024, 3, 2, 1))

        # 4x
        for i in range(4):
            self.d52.add_module(Conv2d(1024, 512, 1, 1, 0))
            self.d52.add_module(Conv2d(512, 1024, 3, 1, 1))
            self.d52.add_module(Residual(1024))

    def forward(self, x):
        return self.d52(x)


class YOLOV3(nn.Module):
    def __init__(self, in_channels, out_channels):
        super(YOLOV3, self).__init__()
        self.darknet52 = Darknet52(in_channels, out_channels)
        self.convSet1=ConvSet(in_channels=1024, )

    def forward(self, x):
        x_52 = self.darknet52(x)



"""
数据分析软件
"""
import pandas as pd
from PyQt5 import QtCore

from yangke.performance.basic_para import Case, alternative_symbol
from yangke.base import extend_list_by_alternative_symbols, get_temp_para, save_temp_para
from yangke.core import str_in_list
from yangke.common.config import logger
import copy
from yangke.common.qt import YkWindow, YkDataTableWidget
from yangke.performance.data_process import suffix_imp_20, suffix_p_20, suffix_dcs_20
from PyQt5.QtWidgets import QApplication, QFileDialog, QProgressBar
from PyQt5.QtGui import QBrush, QRadialGradient
from PyQt5.QtCore import Qt, QObject, pyqtSignal
import sys
import os
from yangke.performance.da import update_data, generate_case_skeleton


class MainWindow(YkWindow, QObject):

    def __init__(self, data_folder=None):
        """
        初始化计算分析软件

        :param data_folder: 数据文件目录，可选参数，不指定时会读取save_temp_para("directory", folder)保存的目录（可能由其他软件保存），
        没有保存记录则为C盘根目录。
        :param
        """
        self.data_folder = data_folder or get_temp_para('directory',
                                                        get_func_or_default_value=lambda: "c:\\")  # 这个和data_process共用一个文件夹
        self.imp_data_file = ""  # imp采集文件，从data_folder中查找"imp汇总_修正.xlsx"文件
        self.dcs_data_file = ""  # 电厂导数数据，从data_folder中查找"dcs汇总_修正.xlsx"文件
        self.power_data_file = ""  # 功率表采集文件，从data_folder中查找"p汇总_修正.xlsx"文件
        # noinspection PyTypeChecker
        self.table_widget: YkDataTableWidget = None
        self.progressBar = QProgressBar()
        self.cases = {}  # 试验工况计算数据列表
        self.case_skeleton = None

        self.q_raidal_grad = QRadialGradient(0, 0, 200, 0, 0)
        self.q_raidal_grad.setColorAt(0, Qt.red)
        self.q_raidal_grad.setColorAt(0.5, Qt.cyan)
        self.q_raidal_grad.setColorAt(1, Qt.yellow)
        super(MainWindow, self).__init__()
        if self.data_folder:
            self.update_file(self.data_folder)

    def update_file(self, folder):
        """
        更新相关文件路径

        :param folder:
        :return:
        """
        self.data_folder = folder
        save_temp_para("directory", folder)
        file = os.path.join(self.data_folder, suffix_imp_20 + ".xlsx")
        if os.path.exists(file):
            self.imp_data_file = file
        else:
            self.imp_data_file = "文件不存在"
            x, y = self.table_widget.get_var_location("采集数据文件")
            if x == -1:
                logger.warning("表格中不存在工作目录变量！")
                return
            self.table_widget.item(x, y - 1).setBackground(QBrush(self.q_raidal_grad))
        file = os.path.join(self.data_folder, suffix_p_20 + ".xlsx")
        if os.path.exists(file):
            self.power_data_file = file
        else:
            self.power_data_file = "文件不存在"
            x, y = self.table_widget.get_var_location("功率数据文件")
            self.table_widget.item(x, y - 1).setBackground(QBrush(self.q_raidal_grad))
        file = os.path.join(self.data_folder, suffix_dcs_20 + ".xlsx")
        if os.path.exists(file):
            self.dcs_data_file = file
        else:
            self.dcs_data_file = "文件不存在"
            x, y = self.table_widget.get_var_location("dcs导数文件")
            self.table_widget.item(x, y - 1).setBackground(QBrush(self.q_raidal_grad))
        self.table_widget.set_value("数据文件目录", folder)
        self.table_widget.set_value("采集数据文件", self.imp_data_file)
        self.table_widget.set_value("功率数据文件", self.power_data_file)
        self.table_widget.set_value("dcs导数文件", self.dcs_data_file)

    def init_ui(self):
        """
        构建界面

        :return:
        """

        def choose_file(table: YkDataTableWidget):
            folder = QFileDialog.getExistingDirectory(parent=self, caption="选择数据存储目录",
                                                      directory=self.data_folder)
            if folder != self.data_folder:
                self.update_file(folder)

        def deal(table: YkDataTableWidget):
            self.progressBar.setVisible(True)
            self.statusBar().showMessage("校验测点数据...")
            self.cases = update_data(imp_file=self.imp_data_file, dcs_file=self.dcs_data_file,
                                     p_file=self.power_data_file)
            # noinspection all
            # self.yk_signal.emit("eval", "self.cases_updated()")
            self.cases_updated()
            # self.progressBar.setValue(0)

        YkDataTableWidget.choose_file = choose_file  # 向YkDataTableWidget添加两个成员方法，以便其中的按钮绑定事件方法
        YkDataTableWidget.deal = deal
        self.table_widget = YkDataTableWidget(from_file="ui/table_data2.yaml")
        self.table_widget.set_value("采集数据文件", self.imp_data_file)
        self.table_widget.set_value("功率数据文件", self.power_data_file)
        self.table_widget.set_value("dcs导数文件", self.dcs_data_file)
        self.setCentralWidget(self.table_widget)
        if self.data_folder is not None and self.data_folder != "c:\\":
            self.table_widget.set_value("数据文件目录", self.data_folder)

        self.statusBar().addPermanentWidget(self.progressBar)
        self.progressBar.setGeometry(0, 0, 50, 2)
        self.progressBar.setValue(0)
        self.progressBar.setVisible(False)
        self.setGeometry(300, 300, 1200, 900)
        self.setWindowTitle("性能试验数据计算")

    def cases_updated(self):
        """
        将self.cases的结果显示出来

        :return:
        """
        if len(self.cases) == 0:
            logger.error("没有找到任何试验工况或试验工况数量为0")
            return None
        _, v = self.cases.popitem()
        self.case_skeleton = generate_case_skeleton(v)

    @QtCore.pyqtSlot(str, str)
    def on_message(self, command, content):
        if command == "eval":
            eval(content)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w1 = MainWindow()
    sys.exit(app.exec_())



# 从性能角度来讲，windows系统使用IOCP、linux使用epoll是高并发情况下最好的实现，他们都是多路IO服用，不是并行框架
# windows IOCP可以使用twisted
# 从简单和通用角度来讲，本模块使用flask+gevent实现，
from gevent import monkey
import socket
import gevent

monkey.patch_all()

server = socket.socket()
server.bind(('0.0.0.0', 8890))
server.listen(1000)


def worker(conn):
    data = conn.recv(1024)
    if data:
        print(data.decode('utf8'))
        conn.send(data)
    else:
        conn.close()


while True:
    conn, addr = server.accept()  # 获得连接
    gevent.spawn(worker, conn)  #
